# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""


#import zipfile  # 解压文件
import shutil  # 用于压缩
import stat,os,time

from PyQt5.QtWidgets import QApplication,QInputDialog,QTreeWidget,QFileIconProvider,QTreeWidgetItem,QLineEdit,QMessageBox,QVBoxLayout,QWidget,QAction,QMenu,QAbstractItemView,QFileDialog

from PyQt5.QtCore import QFileInfo,Qt,QSize,pyqtSignal
from PyQt5.QtGui import QIcon,QCursor, QBrush
from ...conf import res
from ..stdmsg import reso
# 文件监控
import json
from . import stdinit

from watchdog.events import FileSystemEventHandler
import threading
from watchdog.utils.dirsnapshot import DirectorySnapshot, DirectorySnapshotDiff
# from function import process
# dsd = process.ProgressBar_start() #启动第一时间无法保证显示
from watchdog.observers import Observer
import subprocess
from xml.etree import ElementTree as ET
from xml.dom import minidom
import xml.dom.minidom

# 文件夹监控
class StdPathHandler(FileSystemEventHandler,QWidget):
    # trigger = pyqtSignal(int, str)  # 0 settext 1 appead 3process
    my_signal = pyqtSignal(int, str, str)


    def __init__(self):
        super(StdPathHandler, self).__init__()

        try:
            stdinit.std_signal_gobal.project_path_observer.connect(self.project_open_observer)

            stdinit.std_signal_gobal.work_space.connect(self.workspace_change)
            self.brush_red = QBrush(Qt.red)
            self.brush_white = QBrush(Qt.white)
            # self.trigger.emit(1, reso.row_line + str(Digit) + '/' + s2[it:])
            #self.my_signal.connect(parent.my_signal.emit)

            self.dic_name = ""
            self.timer = None
            self.history_active_project=0
            self.observer = Observer()
            self.project_init()
            self.observer_stop_or_working=True






            self.setWindowTitle('Stduino Project')
            self.tree = QTreeWidget(self)

            self.tree.setHeaderHidden(True)
            # self.tree.clicked.connect(self.onTreeClicked)
            self.tree.doubleClicked.connect(self.ondoubleClicked)

            # 给树状文件列表添加右键菜单
            self.tree.setContextMenuPolicy(Qt.CustomContextMenu)  # 开放右键策略
            self.tree.customContextMenuRequested.connect(self.OnTreeRightMenuShow)  # 关联右键槽函数
            self.mainLayout = QVBoxLayout(self)
            #self.init_open()







            # self.Open_Folder()
            # from function.filetree import TreeExplorer
            # file_explorer =TreeExplorer(None)
            # print(os.getcwd())
            # file_explorer.display_directory("C:/stwork/stdemo2019827/dist/Stduino/main/tool","C:/stwork/stdemo2019827/dist/Stduino/main/tool")#os.getcwd())
            # file_explorer.open_file_signal.connect(self.open_file)
            # file_explorer.icon_manipulator.set_icon(
            #     file_explorer,
            #     'tango_icons/system-show-cwd-tree-blue.png'
            #     # functions.create_icon(
            #     #     'tango_icons/system-show-cwd-tree-blue.png'
            #     # )
            # )
            # self.mainLayout.addWidget(file_explorer)#test
            self.setLayout(self.mainLayout)
        except:
            stdinit.std_signal_gobal.stdprintln()

            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile00036！\n You can search it in stduino.com",QMessageBox.Yes)


    def project_open_observer(self,path):
        t1 = threading.Thread(target=self.open_project, name='open_project_p', args=(path,))
        t1.setDaemon(True)
        t1.start()  # File_tree_init

    def project_init(self):
        t1 = threading.Thread(target=self.init_p, name='init_p')
        t1.setDaemon(True)
        t1.start()  # File_tree_init

    def open_project(self,path):

        self.aim_path =path
        self.observer.schedule(self, self.aim_path, True)
        self.observer_stop_or_working = True
        if not self.observer.isAlive():
            self.observer.start()

        self.snapshot = DirectorySnapshot(self.aim_path)

        self.init_open()
        self.add_project_session(path)
        time.sleep(0.05)
        self.workspace_change()





    def close_project_treeview(self):

        self.tree.clear()
        self.observer_stop_or_working=False
        self.close_active_project_session()


        pass
    def project_file_close(self):
        pass
    def XmlToStr(self, elementtree):
        try:
            '''
                    将节点转化成字符串，并添加缩进
                    :param elem:
                    :return:
                    '''

            rough_str = ET.tostring(elementtree, 'utf-8').decode('utf-8','ignore').replace('\n','').replace('  ', '').replace('\t', '').encode('utf-8')
            reparsed = minidom.parseString(rough_str)
            return reparsed.toprettyxml(indent='\t')

            # f = open(self.svd_xml_file_path, 'w', encoding='utf-8')
            # # 保存
            # f.write(new_str)
            # f.close()
            #
            # rough_string = ET.tostring(elementtree, 'utf-8')
            # reparsed = minidom.parseString(rough_string)
            # # 返回缩进
            # return reparsed.toprettyxml(indent="\t")

        except:
            stdinit.std_signal_gobal.stdprintln()


    def init_p(self):
        try:
            # 使用minidom解析器打开 XML 文档
            session = stdinit.session_dir + "/prosession.xml"

            isExists = os.path.exists(session)
            if isExists:

                # DOMTree = xml.dom.minidom.parse(session)
                # self.collection = DOMTree.documentElement
                self.ProjectXmlFileread=ET.parse(session)

                self.ProjectXmltreeRoot=self.ProjectXmlFileread.getroot()






                history_propath = self.ProjectXmltreeRoot.attrib.get('ProjectActivep')
                isExists = os.path.exists(history_propath)
                if isExists:
                    self.aim_path = history_propath  # .replace("/Projects/", "")
                    self.observer.schedule(self, self.aim_path, True)
                    self.observer.start()
                    self.snapshot = DirectorySnapshot(self.aim_path)
                    self.init_open()
                    self.history_active_project = 1
                else:

                    self.tree.clear()
                    self.history_active_project=0


            else:
                self.history_active_project=0
                self.create_project_session()



            #aim_path = stdinit.projects_dir + "407tesst"  # .replace("/Projects/", "")

        except:
            self.history_active_project = 0
            self.create_project_session()
            stdinit.std_signal_gobal.stdprintln()
    def change_active_project_session(self):
        pass
    def close_active_project_session(self):
        self.ProjectXmltreeRoot.attrib['ProjectActivep'] = "None"
        # 保存
        r = self.XmlToStr(self.ProjectXmltreeRoot)
        if os.path.exists(stdinit.session_dir):
            pass
        else:
            os.mkdir(stdinit.session_dir)
        session = stdinit.session_dir + "/prosession.xml"
        f = open(session, 'w+', encoding='utf-8')
        f.write(r)
        f.close()
        pass
    def create_project_session(self):

        self.ProjectXmltreeRoot= ET.Element("Stduinoprojects",
                                               {'ProjectActivep': "None"})  # 创建根节点

        r = self.XmlToStr(self.ProjectXmltreeRoot)

        if os.path.exists(stdinit.session_dir):
            pass
        else:
            os.mkdir(stdinit.session_dir)
        session = stdinit.session_dir + "/prosession.xml"
        f = open(session, 'w+', encoding='utf-8')
        f.write(r)
        f.close()
        pass


    def add_project_session(self,path):
        # 创建子节点
        is_in_xml=0
        for i in self.ProjectXmltreeRoot:
            if i.attrib['projectpath']==path:
                is_in_xml = 1
                self.ProjectXmltreeRoot.attrib['ProjectActivep'] = path
                r = self.XmlToStr(self.ProjectXmltreeRoot)
                if os.path.exists(stdinit.session_dir):
                    pass
                else:
                    os.mkdir(stdinit.session_dir)
                session = stdinit.session_dir + "/prosession.xml"
                f = open(session, 'w+', encoding='utf-8')
                f.write(r)
                f.close()
                break

        if is_in_xml==0:
            son1 = ET.Element('Project', {'projectpath': path})
            self.ProjectXmltreeRoot.append(son1)
            self.ProjectXmltreeRoot.attrib['ProjectActivep'] = path
            # 保存
            r = self.XmlToStr(self.ProjectXmltreeRoot)
            if os.path.exists(stdinit.session_dir):
                pass
            else:
                os.mkdir(stdinit.session_dir)
            session = stdinit.session_dir + "/prosession.xml"
            f = open(session, 'w+', encoding='utf-8')
            f.write(r)
            f.close()
            print(1233233)
            pass





    # def work_sp(self):
    #     self.workspace_change()



    def on_any_event(self, event):
        try:
            if self.observer_stop_or_working:
                if self.timer:
                    self.timer.cancel()
                self.timer = threading.Timer(0.5, self.checkSnapshot)
                self.timer.start()

        except:
            stdinit.std_signal_gobal.stdprintln()


    def dir_created(self, r_path):

        try:
            for path in r_path:

                path = path.replace('\\', '/')
                #print(path)
                fileInfo = QFileInfo(path)
                fileIcon = QFileIconProvider()
                icon = QIcon(fileIcon.icon(fileInfo))
                c_file = path.split("/")
                c_name = c_file[-1]
                c_file[-1] = ""
                new_len = c_file[-2]
                # old_p=c_file.index(self.dic_name)
                postion = c_file.count(new_len) - 1
                ####### f_word = "^" + new_len + "[\w|\W]*"
                # root_item = self.tree.findItems(new_len, Qt.MatchExactly | Qt.MatchRecursive, 0)[postion]  # 第一个0是第几列的意思，本树仅一列，但递归有多个
                #
                # child = QTreeWidgetItem(root_item)
                #
                # # c_name=c_file[-1]
                # child.setText(0, c_name)
                # child.setData(0, Qt.UserRole + 1, path)
                # child.setIcon(0, QIcon(icon))
                # if ".ino" in c_name:
                #
                #     child.setIcon(0, QIcon("appearance/img/img_ino.png"))
                # elif ".cpp" in c_name:
                #
                #     child.setIcon(0, QIcon("appearance/img/img_cpp.png"))
                # elif ".h" in c_name:
                #
                #     child.setIcon(0, QIcon("appearance/img/img_h.png"))
                # elif ".c" in c_name:
                #     child.setIcon(0, QIcon("appearance/img/img_c.png"))
                # elif os.path.isdir(path):
                #     child.setIcon(0, QIcon("appearance/img/img_folder.png"))
                # else:
                #     icon = QIcon(fileIcon.icon(fileInfo))
                #     child.setIcon(0, QIcon(icon))
        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile00037！\n You can search it in stduino.com",QMessageBox.Yes)
    def tags_uploads(self):
        try:
            init_path = stdinit.init_path
            stdinit.pro_conf.clear()
            stdinit.pro_conf.read(init_path, encoding="utf-8")  # python3
            i = 0
            try:
                ii = len(stdinit.pro_conf.sections())

                if ii>0:
                    for i in range(ii):

                        if "env:" in stdinit.pro_conf.sections()[i]:
                            break
                else:
                    return 0
            except:
                stdinit.std_signal_gobal.stdprintln()
                return 0

            try:
                stdinit.framework = stdinit.pro_conf.get(stdinit.pro_conf.sections()[i], "framework")
            except:
                return 0
            try:
                current_boardn = stdinit.pro_conf.get(stdinit.pro_conf.sections()[i], "board")
            except:
                return 0
            try:
                platformnew = stdinit.pro_conf.get(stdinit.pro_conf.sections()[i], "platform")
            except:
                return 0

            # try:
            #     stdinit.last_com = stdinit.pro_conf.get(stdinit.pro_conf.sections()[i], "upload_port")
            # except:
            #     stdinit.last_com = "None"
            #     pass
            try:
                stdinit.current_upload_m = stdinit.pro_conf.get(stdinit.pro_conf.sections()[i], "upload_protocol")
            except:
                stdinit.current_upload_m = None
                pass

            if stdinit.platform == None:
                # print(253)

                if stdinit.framework == "arduino":
                    stdinit.platform = platformnew
                    if stdinit.platform_is == "Win":
                        stdinit.std_signal_gobal.std_jump_init()
                    elif stdinit.platform_is == "Linux":
                        pass
                    elif stdinit.platform_is == "Darwin":
                        pass

            else:
                if stdinit.platform == platformnew:
                    pass
                else:
                    if stdinit.framework == "arduino":
                        stdinit.platform = platformnew
                        if stdinit.platform_is == "Win":
                            stdinit.std_signal_gobal.std_jump_init()
                        elif stdinit.platform_is == "Linux":
                            pass
                        elif stdinit.platform_is == "Darwin":
                            pass

                        # stdinit.std_signal_gobal.std_jump_to_look_iit()
                        # stdinit.goto_init.go_jump_iit()
                        # print(254)
            if stdinit.current_board == current_boardn:
                pass
            else:
                upload_p = stdinit.stdenv + "/.platformio/boards/"  + current_boardn + ".json"
                if os.path.exists(upload_p):
                    stdinit.upload_meds.clear()
                    stdinit.current_board = current_boardn
                    fo = open(upload_p, mode='r', encoding='UTF-8')
                    st = fo.read()
                    fo.close()
                    psd = json.loads(st)
                    try:
                        upload_medd = psd["upload"]["protocol"]
                    except:
                        upload_medd = "Default"
                    try:
                        stdinit.upload_meds = psd["upload"]["protocols"]
                    except:
                        stdinit.upload_meds.append("Default")

                    if stdinit.current_upload_m == None:
                        if upload_medd == "Default":
                            stdinit.current_upload_m = upload_medd
                        else:
                            stdinit.pro_conf.set(stdinit.pro_conf.sections()[i], "upload_protocol", upload_medd)
                            stdinit.current_upload_m = upload_medd
                            stdinit.pro_conf.write(open(init_path, 'w'))
                    stdinit.std_signal_gobal.std_upload_list_change()
                else:
                    upload_p = stdinit.stdenv + "/.platformio/platforms/" + platformnew + "/boards/" + current_boardn + ".json"
                    if os.path.exists(upload_p):
                        stdinit.upload_meds.clear()
                        stdinit.current_board = current_boardn
                        fo = open(upload_p, mode='r', encoding='UTF-8')
                        st = fo.read()
                        fo.close()
                        psd = json.loads(st)
                        try:
                            upload_medd = psd["upload"]["protocol"]
                        except:
                            upload_medd = "Default"
                        try:
                            stdinit.upload_meds = psd["upload"]["protocols"]
                        except:
                            stdinit.upload_meds.append("Default")

                        if stdinit.current_upload_m == None:
                            if upload_medd == "Default":
                                stdinit.current_upload_m = upload_medd
                            else:
                                stdinit.pro_conf.set(stdinit.pro_conf.sections()[i], "upload_protocol", upload_medd)
                                stdinit.current_upload_m = upload_medd
                                stdinit.pro_conf.write(open(init_path, 'w'))

                        stdinit.std_signal_gobal.std_upload_list_change()
                    pass


        except:
            stdinit.std_signal_gobal.stdprintln()

    def workspace_change(self):

        try:
            work_path = stdinit.projects_dir
            old_workfpace = stdinit.pro_dir_name
            new_workspace =os.path.basename(self.aim_path)


            # 开启新工作空间
            #print("new_workspace"+new_workspace)
            #print("old_workfpace " + old_workfpace )
            if old_workfpace == new_workspace:
                return 0

            root_item = self.tree.findItems(new_workspace, Qt.MatchExactly | Qt.MatchRecursive, 0)
            f_len = len(root_item)
            # print(c_path)
            root_item_id = None
            #print(f_len)

            if f_len > 1:
                #print("new2"+new_workspace)
                for i in range(f_len):
                    if work_path + new_workspace == root_item[i].data(0, Qt.UserRole + 1):
                        root_item_id = root_item[i]
                        # print(root_item[i].data(0, Qt.UserRole + 1))
                pass
            elif f_len == 1:
                #print("new1" + new_workspace)
                if work_path + new_workspace == root_item[0].data(0, Qt.UserRole + 1):
                    root_item_id = root_item[0]
                    pass
            else:
                #print("new33" + new_workspace)
                return 0
            #print("new3" + new_workspace)
            if root_item_id != None and stdinit.project_name!=None:
                dir_is=stdinit.projects_dir + stdinit.project_name
                if os.path.isdir(dir_is):
                    # root_item_id.setBackground(0,brush_red)
                    root_item_id.setForeground(0, self.brush_red)
                    root_item_id.setToolTip(0, "Working~")
                    root_item_id.setIcon(0, QIcon("appearance/img/run.png"))

                    try:
                        stdinit.init_path = stdinit.projects_dir + stdinit.project_name + "/platformio.ini"
                        if os.path.isfile(stdinit.init_path):
                            pass
                        else:
                            stdinit.init_path = stdinit.projects_dir + stdinit.project_name + "/stduino.ini"
                            pass

                        if os.path.exists(stdinit.init_path):
                            t1 = threading.Thread(target=self.tags_uploads, name=stdinit.project_name)
                            t1.setDaemon(True)
                            t1.start()  # File_tree_init
                            # platformnew = stdinit.pro_conf.get(stdinit.pro_conf.sections()[0], "platform")
                                    #
                        else:
                            stdinit.init_path = None
                    except:
                        stdinit.std_signal_gobal.stdprintln()

                else:
                    return 0

                # 关闭之前工作空间
            if old_workfpace != None:
                root_item = self.tree.findItems(old_workfpace, Qt.MatchExactly | Qt.MatchRecursive, 0)
                stdinit.pro_dir_name = stdinit.project_name
                f_len = len(root_item)
                # print(c_path)
                root_item_id = None

                if f_len > 1:
                    for i in range(f_len):
                        if work_path + old_workfpace == root_item[i].data(0, Qt.UserRole + 1):
                            root_item_id = root_item[i]
                            # print(root_item[i].data(0, Qt.UserRole + 1))
                    pass
                elif f_len == 1:
                    if work_path + old_workfpace == root_item[0].data(0, Qt.UserRole + 1):
                        root_item_id = root_item[0]
                        pass

                else:

                    return 0
                if root_item_id != None:
                    # root_item_id.setBackground(0,brush_red)
                    root_item_id.setForeground(0, self.brush_white)
                    root_item_id.setToolTip(0, "")
                    # root_item_id.setIcon(0, QIcon("appearance/img/img_folder.png"))
                    if ".ino" in old_workfpace:
                        root_item_id.setIcon(0, QIcon("appearance/img/img_ino.png"))
                    elif ".cpp" in old_workfpace:

                        root_item_id.setIcon(0, QIcon("appearance/img/img_cpp.png"))
                    elif ".h" in old_workfpace:

                        root_item_id.setIcon(0, QIcon("appearance/img/img_h.png"))
                    elif ".c" in old_workfpace:
                        root_item_id.setIcon(0, QIcon("appearance/img/img_c.png"))
                    elif ".ini" in old_workfpace:
                        root_item_id.setIcon(0, QIcon("appearance/img/piofavicon.svg"))
                    else:
                        root_item_id.setIcon(0, QIcon("appearance/img/img_folder.png"))
                    # root_item_id.setCheckState(0, Qt.Unchecked)
            stdinit.pro_dir_name = stdinit.project_name
            #print("new_workspac121e" + new_workspace)

        except:
            stdinit.std_signal_gobal.stdprintln()
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容

    def file_created(self, r_path):
        try:
            r_path = sorted(r_path, key=lambda i: len(i), reverse=False)

            for path in r_path:
                path = path.replace('\\', '/')
                #print(path)
                fileInfo = QFileInfo(path)
                fileIcon = QFileIconProvider()
                #icon = QIcon(fileIcon.icon(fileInfo))
                c_file = path.split("/")
                c_name = c_file[-1]
                c_file[-1] = ""
                #print(len(c_file))
                #fnew_len = c_file[-python3]  # 父
                new_len = c_file[-2]#父
                #print(new_len)
                c_path = path.replace("/"+c_name,"")
                # old_p=c_file.index(self.dic_name)
               # postion = c_file.count(new_len) - 1

                ####### f_word = "^" + new_len + "[\w|\W]*"
                root_item = self.tree.findItems(new_len, Qt.MatchExactly | Qt.MatchRecursive, 0)


                f_len=len(root_item)
                #print(c_path)
                root_item_id=None


                if f_len>1:
                    for i in range(f_len):
                        if c_path==root_item[i].data(0, Qt.UserRole + 1):
                            root_item_id=root_item[i]
                            #print(root_item[i].data(0, Qt.UserRole + 1))
                    pass
                elif f_len==1:
                    if c_path == root_item[0].data(0, Qt.UserRole + 1):
                        root_item_id = root_item[0]
                        pass

                else:
                    return 0
                if root_item_id!=None:
                    child = QTreeWidgetItem(root_item_id)
                    # child.setCheckState(0,Qt.Checked)

                    # c_name=c_file[-1]
                    child.setText(0, c_name)
                    child.setData(0, Qt.UserRole + 1, path)

                    # child.setIcon(0, QIcon(icon))
                    if ".ino" in c_name:
                        child.setIcon(0, QIcon("appearance/img/img_ino.png"))
                    elif ".cpp" in c_name:

                        child.setIcon(0, QIcon("appearance/img/img_cpp.png"))
                    elif ".h" in c_name:

                        child.setIcon(0, QIcon("appearance/img/img_h.png"))
                    elif ".c" in c_name:
                        child.setIcon(0, QIcon("appearance/img/img_c.png"))
                    elif ".ini" in c_name:
                        child.setIcon(0, QIcon("appearance/img/piofavicon.svg"))
                    elif os.path.isdir(path):
                        child.setIcon(0, QIcon("appearance/img/img_folder.png"))
                    else:
                        icon = QIcon(fileIcon.icon(fileInfo))
                        child.setIcon(0, QIcon(icon))


                #print(root_item[0].parent().data(0, Qt.UserRole + 1))
                #return 0

                #root_item = self.tree.findItems(new_len, Qt.MatchExactly | Qt.MatchRecursive, 0)[postion]  # 第一个0是第几列的意思，本树仅一列，但递归有多个


        except:
            stdinit.std_signal_gobal.stdprintln()
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile00038！\n You can search it in stduino.com",QMessageBox.Yes)
        # print()

    def file_deleted(self, r_path):
        try:
            r_path = sorted(r_path, key=lambda i: len(i), reverse=True)
            for path in r_path:

                #print(path)
                path = path.replace('\\', '/')
                #print("del"+path)

                c_file = path.split("/")
                new_len = c_file[-1]

                root_item = self.tree.findItems(new_len, Qt.MatchExactly | Qt.MatchRecursive,0)  # [0]  # 第一个0是第几列的意思，本树仅一列，但递归有多个
                for node in root_item:
                    node_text = node.data(0, Qt.UserRole + 1)
                    if node_text == path:
                        self.deleteItem(node)
                        #self.my_signal.emit(6, node_text, node_text)

        except:

            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile00039！\n You can search it in stduino.com",QMessageBox.Yes)



    def file_moved(self, files_moved):
        try:
            del_source = []
            cre_target = []

            for i in files_moved:
                del_source.append(i[0])
                cre_target.append(i[1])
                self.my_signal.emit(5, i[0].replace("\\", "/"), i[1].replace("\\", "/"))

            self.file_deleted(del_source)
            self.file_created(cre_target)

        except:
            stdinit.std_signal_gobal.stdprintln()


        # print()

    def dir_moved(self, files_moved):
        # print(str(files_moved))
        # print(2)
        try:
            # r_path = sorted(files_moved, key=lambda i: len(i), reverse=True)
            # for path in r_path:
            #     #path = path.replace('\\', '/')
            #     print(path)
            # return 0
            del_source=[]
            cre_target=[]

            for i in files_moved:
                del_source.append(i[0])
                cre_target.append(i[1])

            self.file_deleted(del_source)
            self.file_created(cre_target)
        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0041！\n You can search it in stduino.com",QMessageBox.Yes)

    def file_modify(self,modify_list):
        try:
            modify_ui=stdinit.profile_modify_ui
            modify_all=stdinit.profile_modify_all
            for i in modify_list:
                ii=i.replace('\\', '/')
                if ii in modify_all:
                    pass
                else:
                    stdinit.profile_modify_all.append(ii)
            modify_all = stdinit.profile_modify_all
            for i in modify_ui:
                if i in modify_all:
                    stdinit.profile_modify_all.remove(i)
                    stdinit.profile_modify_ui.remove(i)
            if len(stdinit.profile_modify_all)>0:
                stdinit.std_signal_gobal.std_file_modify()

        except:
            stdinit.std_signal_gobal.stdprintln()

    def checkSnapshot(self):
        try:

            snapshot = DirectorySnapshot(self.aim_path)
            diff = DirectorySnapshotDiff(self.snapshot, snapshot)
            self.snapshot = snapshot
            self.timer = None

            # print("files_created:", diff.files_created)
            # print("files_deleted:", diff.files_deleted)

            # print("files_moved:", diff.files_moved)
            # print("dirs_modified:", diff.dirs_modified)
            # print("dirs_moved:", diff.dirs_moved)
            # print("dirs_deleted:", diff.dirs_deleted)
            # print("dirs_created:", diff.dirs_created)
            if diff.files_deleted:

                self.file_deleted(diff.files_deleted)
            if diff.dirs_deleted:

                self.file_deleted(diff.dirs_deleted)
            if diff.dirs_moved:

                self.dir_moved(diff.dirs_moved)

            if diff.files_moved:

                self.file_moved(diff.files_moved)

            if diff.dirs_created:

                self.file_created(diff.dirs_created)
            if diff.files_created:

                self.file_created(diff.files_created)
            if diff.files_modified:
                self.file_modify(diff.files_modified)
                #print("files_modified:", diff.files_modified)

                # print("cre b"+diff.dirs_created)
                # print("cre f"+diff.files_created)




            # 调试变动输出

            # 接下来就是你想干的啥就干点啥，或者该干点啥就干点啥
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0042！\n You can search it in stduino.com",QMessageBox.Yes)


    # 节点点击事件
    def ondoubleClicked(self):
        try:
            item1 = self.tree.currentItem()
            item = item1.data(0, Qt.UserRole + 1)

            if os.path.isdir(item):
                #d_name=item1.text(0)
                c_file = item.split("/")

                if len(c_file)>5:
                    if c_file[4] == "Projects":
                        stdinit.project_name = c_file[5]
                        self.workspace_change()
                    if c_file[5] == "Projects":
                        stdinit.project_name = c_file[6]
                        self.workspace_change()


                # if os.path.isdir(stdinit.projects_dir+d_name):
                #     stdinit.project_name = d_name
                #     self.workspace_change()
                # pass
            else:

                self.my_signal.emit(1, item1.text(0), item)
        except:
            stdinit.std_signal_gobal.stdprintln()
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0043！\n You can search it in stduino.com",QMessageBox.Yes)


    def onTreeClicked(self):
        try:
            item = self.tree.currentItem()
            D = item.data(0, Qt.UserRole + 1)
            #print(D)
        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0044！\n You can search it in stduino.com",QMessageBox.Yes)

        # print("key=%s " % (item.text(0)))

    # 删除控件树子节点/根节点
    def deleteItem(self, currNode):

        try:
            # 尝试删除子节点（通过其父节点，调用removeChild函数进行删除）

            parent1 = currNode.parent()
            parent1.removeChild(currNode)
        except Exception:
            # 遇到异常时删除根节点
            try:
                rootIndex = self.tree.indexOfTopLevelItem(currNode)
                self.tree.takeTopLevelItem(rootIndex)
            except Exception:
                stdinit.std_signal_gobal.stdprintln()

    # 添加节点
    def addTreeNodeBtn(self):
        #print('--- addTreeNodeBtn ---')
        item = self.tree.currentItem()

        node = QTreeWidgetItem(item)
        node.setText(0, 'newNode')
        node.setText(1, '10')
    def paste_fun(self,f):
        try:
            try:
                currentItem_object = self.tree.currentItem()  # 获取鼠标所在的树状列表的项
                # currentItem_object_text = currentItem_object.text(0)
                b = currentItem_object.data(0, Qt.UserRole + 1)  # 获取鼠标所在的树状列表的项path
                currentItem_parent_path = currentItem_object.parent().data(0, Qt.UserRole + 1)  # 获取鼠标所在的树状列表的项
                if os.path.isdir(b):
                    pass
                else:
                    b = currentItem_parent_path
            except:
                return 0

            f_path_name = f.split("/")[-1]
            topath1 = b + "/" + f_path_name
            # f_path_name = b + "/" + f_path_name
            if f in b:
                QMessageBox.warning(self,
                                    "禁止行为",
                                    "该步操作将不进行执行！",
                                    QMessageBox.Yes)
                return 0
            elif os.path.exists(topath1):
                reply = QMessageBox.warning(self,
                                            "文件\夹已存在",
                                            "文件\夹已存在,是否覆盖！",
                                            QMessageBox.Yes | QMessageBox.No)
                if reply == QMessageBox.No:
                    return 0

            # _thread.start_new_thread(self.paste_fun(f, b))

            t1 = threading.Thread(target=self.paste_fun1, name='file1', args=(f, b,))
            t1.setDaemon(True)
            t1.start()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def paste_fun1(self,f_path,topath):

        try:

            # self.pb12.setVisible(False)
            # the_window.pb1_2.setVisible(True)
            # the_window.pb1_2.setText(reso.downloading)
            stdinit.std_signal_gobal.std_process(1,"开始拷贝")

            if os.path.isdir(f_path):
                # if f_path in topath:
                #     QMessageBox.warning(self,
                #                         "禁止行为",
                #                         "该步操作将不进行执行！",
                #                         QMessageBox.Yes)
                #     return 0

                f_path_name = f_path.split("/")[-1]
                topath = topath + "/" + f_path_name
                if os.path.exists(topath):
                #     reply = QMessageBox.warning(self,
                #                                 "文件夹已存在",
                #                                 "是否覆盖！",
                #                                 QMessageBox.Yes | QMessageBox.No)
                #     if reply == QMessageBox.No:
                #         return 0
                    #the_window.pb12.setValue(65)

                    shutil.rmtree(topath)
                    shutil.copytree(f_path, topath)

                    #the_window.pb12.setValue(100)

                else:

                    shutil.copytree(f_path, topath)  # 每次会新建一个文件夹 被复制的仅仅是内容放至新建文件夹里




            #elif os.path.isfile(f_path):
            else:

                f_path_name = f_path.split("/")[-1]
                f_path_name = topath + "/" + f_path_name
                if os.path.exists(f_path_name):
                    #the_window.pb12.setValue(80)
                    os.remove(f_path_name)
                    #the_window.pb12.setValue(90)
                    shutil.copy2(f_path, topath)
                    #the_window.pb12.setValue(100)
                else:
                    pass
                    #the_window.pb12.setValue(80)
                    shutil.copy2(f_path, topath)
                    #the_window.pb12.setValue(100)

            stdinit.std_signal_gobal.std_process(0, "拷贝完成")
            #the_window.pb12.setVisible(False)

        except:
            stdinit.std_signal_gobal.std_echo_msg(1,"粘贴失败，请手动进入文件夹粘贴！\nPaste failed, please go to the folder manually and paste!")
            #self.my_paste.emit(0, 100,"粘贴失败，请手动进入文件夹粘贴！\nPaste failed, please go to the folder manually and paste!")
            #the_window.pb12.setVisible(False)
            # QMessageBox.warning(self,
            #                     "Paste failed",
            #                     "粘贴失败，请手动进入文件夹粘贴！\nPaste failed, please go to the folder manually and paste!",
            #                     QMessageBox.Yes)
            # pass




    #     pass
    def NewFileMenuShow(self):  # 当点击“source”和“header” 显示添加文件菜单；点击具体文件名时，显示删除文件菜单
        try:
            self.tree.popMenu = QMenu()
            add = QMenu(reso.add_file)
            add.setIcon(QIcon(res.img_addfile))
            #
            # addFile = QAction(,, self)
            addFile_ino = QAction(QIcon("appearance/img/img_ino.png"), ".ino File", self)
            addFile_cpp = QAction(QIcon("appearance/img/img_cpp.png"), ".c++ File", self)
            addFile_h = QAction(QIcon("appearance/img/img_h.png"), ".h File", self)
            addFile_c = QAction(QIcon("appearance/img/img_c.png"), ".c File", self)
            addFile_txt = QAction(QIcon("appearance/img/img_txt.png"), " File", self)
            adddirec = QAction(QIcon("appearance/img/img_folder.png"), reso.add_direc, self)

            add.addAction(adddirec)
            add.addAction(addFile_ino)
            add.addAction(addFile_cpp)
            add.addAction(addFile_c)
            add.addAction(addFile_h)
            add.addAction(addFile_txt)

            # 1open 2rename python3 delete 4 new
            addFile_ino.triggered.connect(lambda: self.new_file(".ino"))  # (lambda:self.toggleMenu3(1))
            addFile_h.triggered.connect(lambda: self.new_file(".h"))
            addFile_c.triggered.connect(lambda: self.new_file(".c"))
            addFile_cpp.triggered.connect(lambda: self.new_file(".cpp"))
            addFile_txt.triggered.connect(lambda: self.new_file(""))
            adddirec.triggered.connect(lambda: self.new_file("folder"))
            add.exec_(QCursor.pos())  # 在鼠标位置显示

        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0045！\n You can search it in stduino.com",QMessageBox.Yes)
    def remove_readonly(self, func, path, _):
        "Clear the readonly bit and reattempt the removal"
        os.chmod(path, stat.S_IWRITE)
        func(path)

    def delete_file(self, path):
        try:
            if os.path.isdir(path):
                try:
                    shutil.rmtree(path, onerror=self.remove_readonly)
                except:
                    stdinit.std_signal_gobal.std_echo_msg(1,
                                                          "Delete error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")
                    # self.add_lib_si(python3, 0, "Unexpected error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")
                    # QMessageBox.warning(self, "Delete error",
                    #                     "Unexpected error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder",
                    #                     QMessageBox.Yes)
                    pass
            else:
                try:
                    os.remove(path)
                except:
                    try:
                        # shutil.rmtree(path, onerror=self.remove_readonly)
                        os.chmod(path, stat.S_IWRITE)
                        os.remove(path)
                    except:
                        # print("Unexpected error:", sys.exc_info()[1])  # 错误内容
                        stdinit.std_signal_gobal.std_echo_msg(1, "Delete error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")

        except:
            stdinit.std_signal_gobal.stdprintln()



    def new_me(self, i_type):

        try:
            currentItem_object = self.tree.currentItem()  # 获取鼠标所在的树状列表的项
            currentItem_object_text =currentItem_object.text(0)
            currentItem_object_path=currentItem_object.data(0, Qt.UserRole + 1)  # 获取鼠标所在的树状列表的项path
            if currentItem_object_path==stdinit.stdenv+"/Documents/Stduino":
                currentItem_parent_path =currentItem_object_path
            else:
                currentItem_parent_path = currentItem_object.parent().data(0, Qt.UserRole + 1)  # 获取鼠标所在的树状列表的项

        except:
            stdinit.std_signal_gobal.stdprintln()
            return 0

        try:

            if i_type == 2:
                value, ok = QInputDialog.getText(self, "重命名", "输入文件名:", QLineEdit.Normal, currentItem_object_text)
                if ok == 1:
                    try:
                        os.rename(currentItem_object_path, currentItem_parent_path + "/" + value)
                    except:
                        pathn =currentItem_object_path.replace(currentItem_object_text, value)
                        os.rename(currentItem_object_path, pathn)
            elif i_type == 3:
                try:

                    t1 = threading.Thread(target=self.delete_file, name='fielde1', args=(currentItem_object_path,))
                    t1.setDaemon(True)
                    t1.start()
                except:
                    stdinit.std_signal_gobal.std_echo_msg(1, "Delete error",
                                        "Unexpected error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")
                    pass
            elif i_type == 7:

                if os.path.isdir(currentItem_object_path):
                    #os.startfile()

                    if stdinit.platform_is == "Win":
                        os.startfile(currentItem_object_path)

                    elif stdinit.platform_is == "Linux":
                        os.system('xdg-open "%s"' % currentItem_object_path)


                    elif stdinit.platform_is == "Darwin":
                        subprocess.call(["open", currentItem_object_path])



                else:
                    if stdinit.platform_is == "Win":
                        os.startfile(currentItem_parent_path)

                    elif stdinit.platform_is == "Linux":
                        os.system('xdg-open "%s"' % currentItem_parent_path)


                    elif stdinit.platform_is == "Darwin":
                        subprocess.call(["open", currentItem_parent_path])

            elif i_type==8:
                clipboard = QApplication.clipboard()
                clipboard.setText(currentItem_object_path)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def new_file(self,i_name):
        try:
            try:
                currentItem_object = self.tree.currentItem()  # 获取鼠标所在的树状列表的项
                # currentItem_object_text = currentItem_object.text(0)

                currentItem_object_path = currentItem_object.data(0, Qt.UserRole + 1)  # 获取鼠标所在的树状列表的项path
                if currentItem_object_path == stdinit.stdenv + "/Documents/Stduino":
                    currentItem_parent_path = currentItem_object_path
                else:
                    currentItem_parent_path = currentItem_object.parent().data(0, Qt.UserRole + 1)  # 获取鼠标所在的树状列表的项

                if os.path.isdir(currentItem_object_path):
                    pass
                else:
                    currentItem_object_path = currentItem_parent_path

            except:
                stdinit.std_signal_gobal.stdprintln()
                return 0
            value, ok = QInputDialog.getText(self, "新建", "输入文件名:", QLineEdit.Normal, "")
            if ok == 1:
                if i_name == "folder":

                    if os.path.exists(currentItem_object_path + "/" + value):
                        stdinit.std_signal_gobal.std_echo_msg(1, "The folder already exists")

                        pass
                    else:
                        os.makedirs(currentItem_object_path + "/" + value)

                    pass
                else:
                    if os.path.exists(currentItem_object_path + "/" + value + i_name):

                        stdinit.std_signal_gobal.std_echo_msg(1, "The file already exists")
                        pass
                    else:

                        p_name = currentItem_object_path + "/" + value + i_name
                        fo = open(p_name, "w+", encoding='UTF-8')
                        fo.write("")
                        fo.close()
                        self.my_signal.emit(4, p_name, "")
        except:
            stdinit.std_signal_gobal.stdprintln()



    def OnTreeRightMenuShow(self):  # 当点击“source”和“header” 显示添加文件菜单；点击具体文件名时，显示删除文件菜单
        try:
            self.tree.popMenu = QMenu()
            add = QMenu(reso.add_file)
            add.setIcon(QIcon(res.img_addfile))
            #
            # addFile = QAction(,, self)
            addFile_ino = QAction(QIcon("appearance/img/img_ino.png"), ".ino File", self)
            addFile_cpp = QAction(QIcon("appearance/img/img_cpp.png"), ".c++ File", self)
            addFile_h = QAction(QIcon("appearance/img/img_h.png"), ".h File", self)
            addFile_c = QAction(QIcon("appearance/img/img_c.png"), ".c File", self)
            addFile_txt = QAction(QIcon("appearance/img/img_txt.png"), " File", self)
            adddirec = QAction(QIcon("appearance/img/img_folder.png"), reso.add_direc, self)
            opendirec = QAction(QIcon(res.img_open), reso.open_folder, self)



            copy_action = QAction(QIcon("appearance/img/copy.png"), "Copy", self)
            #cut_action = QAction(QIcon("appearance/img/img_folder.png"), "Cut", self)
            paste_action=QAction(QIcon("appearance/img/paste.png"), "Paste", self)
            clipboard = QApplication.clipboard().text()
            path=clipboard.replace("file:///","")

            # for line in path:
            #     print(1)
            #     print(line)

            # ds.replace()
            #
            if os.path.isdir(path) or os.path.isfile(path):
                paste_action.setEnabled(True)
            else:
                paste_action.setEnabled(False)


            rename = QAction(reso.re_name, self)
            delete = QAction(reso.delete_f_d, self)
            add.addAction(adddirec)
            add.addAction(addFile_ino)
            add.addAction(addFile_cpp)
            add.addAction(addFile_c)
            add.addAction(addFile_h)
            add.addAction(addFile_txt)

            item1_text = self.tree.currentItem().text(0)  # 获取鼠标所在的树状列表的项


            # self.tree.popMenu.addAction(addFile)

            if item1_text == self.dic_name:

                self.tree.popMenu.addMenu(add)

                self.tree.popMenu.addAction(opendirec)
                self.tree.popMenu.addAction(copy_action)
                # self.tree.popMenu.addAction(cut_action)
                self.tree.popMenu.addAction(paste_action)
            else:
                self.tree.popMenu.addMenu(add)
                self.tree.popMenu.addAction(opendirec)
                self.tree.popMenu.addAction(rename)
                self.tree.popMenu.addAction(delete)
                self.tree.popMenu.addAction(copy_action)
                # self.tree.popMenu.addAction(cut_action)
                self.tree.popMenu.addAction(paste_action)

            # addFile.addAction(delete)

            # downtype.addAction(self.usblink)
            # downtype.addAction(self.stlink)
            # fileMenu = menubar.addMenu(reso.tool)
            # fileMenu.addAction(self.serialAction)
            # fileMenu.addAction(driver)
            # fileMenu.addMenu(downtype)

            #if os.path.isdir(path):

            # 1open 2rename python3 delete 4 new
            addFile_ino.triggered.connect(lambda: self.new_file(".ino"))  # (lambda:self.toggleMenu3(1))
            addFile_h.triggered.connect(lambda: self.new_file(".h"))
            addFile_c.triggered.connect(lambda: self.new_file(".c"))
            addFile_cpp.triggered.connect(lambda: self.new_file(".cpp"))
            addFile_txt.triggered.connect(lambda: self.new_file(""))
            adddirec.triggered.connect(lambda: self.new_file("folder"))

            opendirec.triggered.connect(lambda: self.new_me(7))

            # adddirec.triggered.connect(self.do_adddirec)
            rename.triggered.connect(lambda: self.new_me(2))
            delete.triggered.connect(lambda: self.new_me(3))
            paste_action.triggered.connect(lambda: self.paste_fun(path))
            #paste_action.triggered.connect(_thread.start_new_thread(self.paste_fun(path, item1)))



            copy_action.triggered.connect(lambda: self.new_me(8))#_thread.start_new_thread(self.startgdbserver, ("Thread-1", 1,))
            self.tree.popMenu.exec_(QCursor.pos())  # 在鼠标位置显示
        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0045！\n You can search it in stduino.com",QMessageBox.Yes)



        # if '.' in item.text(0) and '.cbp' not in item.text(0):  # 该项是文件名，添加右键[删除菜单]



    # 节点更新
    def updateTreeNodeBtn(self):
        print('--- updateTreeNodeBtn ---')
        item = self.tree.currentItem()
        item.setText(0, 'updateNode')
        item.setText(1, '20')

    # 删除节点
    def delTreeNodeBtn(self):
        print('--- delTreeNodeBtn ---')
        item = self.tree.currentItem()
        root = self.tree.invisibleRootItem()
        for item in self.tree.selectedItems():
            (item.parent() or root).removeChild(item)

    def Open_Folder(self):
        try:
            path = QFileDialog.getExistingDirectory(self, "选取文件夹", "./")

            self.tree.setColumnCount(1)
            self.tree.setColumnWidth(0, 50)
            self.tree.setHeaderLabels(["Stduino"])
            self.tree.setIconSize(QSize(25, 25))
            self.tree.setSelectionMode(QAbstractItemView.ExtendedSelection)
            # self.actionfileopen.triggered.connect(self.Open_Folder)
            self.aim_path = path  # aim_path不可随意更换否则容易比对出错
            # print(path)

            dirs = os.listdir(path)

            fileInfo = QFileInfo(path)
            fileIcon = QFileIconProvider()
            icon = QIcon(fileIcon.icon(fileInfo))
            root = QTreeWidgetItem(self.tree)
            # root.setText(0, "projects")
            self.dic_name = path.split('/')[-1]
            root.setText(0, self.dic_name)
            root.setData(0, Qt.UserRole + 1, path)
            # C:\Users\debug\Desktop\ts  #path.split('/')[-1]
            root.setIcon(0, QIcon("appearance/img/img_p.png"))
            self.CreateTree(dirs, root, path)
            # self.setCentralWidget(self.tree)
            #self.mainLayout.addWidget(self.tree)
            QApplication.processEvents()
        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0046！\n You can search it in stduino.com",QMessageBox.Yes)


    def init_open(self):
        try:
            self.tree.clear()
            self.tree.setColumnCount(1)
            self.tree.setColumnWidth(0, 50)
            self.tree.setHeaderLabels(["Stduino"])
            self.tree.setIconSize(QSize(25, 25))
            self.tree.setSelectionMode(QAbstractItemView.ExtendedSelection)
            # self.actionfileopen.triggered.connect(self.Open_Folder)
            # path = r"C:\Users\debug\Desktop\ts"
            # self.aim_path= r"C:\Users\debug\Desktop\ts"
            path = self.aim_path

            dirs = os.listdir(path)

            fileInfo = QFileInfo(path)
            fileIcon = QFileIconProvider()
            icon = QIcon(fileIcon.icon(fileInfo))
            root = QTreeWidgetItem(self.tree)
            self.dic_name = path.split('/')[-1]
            root.setText(0, self.dic_name)
            # C:\Users\debug\Desktop\ts  #path.split('/')[-1]
            root.setData(0, Qt.UserRole + 1, path)
            # root.setIcon(0, QIcon(icon))
            root.setIcon(0, QIcon("appearance/img/img_p4.png"))
            root.setExpanded(True)

            #self.CreateTree(dirs, root, path)
            t1 = threading.Thread(target=self.CreateTree, name='dir1',args=(dirs, root, path,))
            t1.start()
            # self.setCentralWidget(self.tree)
            self.mainLayout.addWidget(self.tree)
            QApplication.processEvents()


        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0047！\n You can search it in stduino.com",QMessageBox.Yes)
        # path = QFileDialog.getExistingDirectory(self, "选取文件夹", "./")
        # self.tree = QTreeWidget()
    def CreateTree1(self, dirs, root, path):

        try:
            # pro_fo=0
            for i in dirs:
                path_new = path + '/' + i
                # print(path_new)
                if os.path.isdir(path_new):
                    # fileInfo = QFileInfo(path_new)
                    # fileIcon = QFileIconProvider()
                    # icon = QIcon(fileIcon.icon(fileInfo))
                    child = QTreeWidgetItem(root)
                    child.setText(0, i)
                    child.setData(0, Qt.UserRole + 1, path_new)
                    child.setIcon(0, QIcon("appearance/img/img_folder.png"))
                    # if pro_fo==0:
                    #     pro_fo=1
                    #     child.setExpanded(True)

                    dirs_new = os.listdir(path_new)
                    self.CreateTree1(dirs_new, child, path_new)
                else:
                    fileInfo = QFileInfo(path_new)
                    fileIcon = QFileIconProvider()
                    child = QTreeWidgetItem(root)
                    child.setText(0, i)
                    child.setData(0, Qt.UserRole + 1, path_new)

                    if ".ino" in i:
                        child.setIcon(0, QIcon("appearance/img/img_ino.png"))#
                    elif ".cpp" in i:
                        child.setIcon(0, QIcon("appearance/img/img_cpp.png"))
                    elif ".h" in i:
                        child.setIcon(0, QIcon("appearance/img/img_h.png"))
                    elif ".c" in i:
                        child.setIcon(0, QIcon("appearance/img/img_c.png"))
                    elif ".ini" in i:
                        child.setIcon(0, QIcon("appearance/img/piofavicon.svg"))
                    else:

                        icon = QIcon(fileIcon.icon(fileInfo))
                        child.setIcon(0, QIcon(icon))


        except:
            stdinit.std_signal_gobal.stdprintln()
           # QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0048！\n You can search it in stduino.com",QMessageBox.Yes)



    def CreateTree(self, dirs, root, path):

        try:
            pro_fo=0
            for i in dirs:
                path_new = path + '/' + i
                # print(path_new)
                if os.path.isdir(path_new):
                    # fileInfo = QFileInfo(path_new)
                    # fileIcon = QFileIconProvider()
                    # icon = QIcon(fileIcon.icon(fileInfo))
                    child = QTreeWidgetItem(root)
                    child.setText(0, i)
                    child.setData(0, Qt.UserRole + 1, path_new)
                    child.setIcon(0, QIcon("appearance/img/img_folder.png"))
                    if pro_fo==0:
                        pro_fo=1
                        child.setExpanded(True)

                    dirs_new = os.listdir(path_new)
                    self.CreateTree1(dirs_new, child, path_new)
                else:
                    fileInfo = QFileInfo(path_new)
                    fileIcon = QFileIconProvider()
                    child = QTreeWidgetItem(root)
                    child.setText(0, i)
                    child.setData(0, Qt.UserRole + 1, path_new)

                    if ".ino" in i:
                        child.setIcon(0, QIcon("appearance/img/img_ino.png"))#
                    elif ".cpp" in i:
                        child.setIcon(0, QIcon("appearance/img/img_cpp.png"))
                    elif ".h" in i:
                        child.setIcon(0, QIcon("appearance/img/img_h.png"))
                    elif ".c" in i:
                        child.setIcon(0, QIcon("appearance/img/img_c.png"))
                    elif ".ini" in i:
                        child.setIcon(0, QIcon("appearance/img/piofavicon.svg"))
                    else:

                        icon = QIcon(fileIcon.icon(fileInfo))
                        child.setIcon(0, QIcon(icon))

            stdinit.File_tree_view.workspace_change()
            #stdinit.std_signal_gobal.std_echo_msg(0, "Stduino IDE初始化完毕，欢迎您的使用，若在使用过程中遇到任何问题，请至www.stduino.com进行发帖交流！")


        except:
            stdinit.std_signal_gobal.stdprintln()
            #QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile0048！\n You can search it in stduino.com",QMessageBox.Yes)




# -*- coding: utf-8 -*-

"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""

import subprocess
import os
import json
from shutil import copy2
from ..cores.stdedit import stdinit
#from function.cores.stdedit import stdinit
#from urllib import request#
import time

#w806=[{"id":"W806","name":"W806duino Nano","platform":"W806","platforms": "stduino","mcu":"xt804","fcpu":240000000,"ram":1048576,"rom":294912,"frameworks":["arduino","hal","freertos"],"vendor":"W806duino","url":"https://gitee.com/stduino/arduino_core_w806.git","debug":{"tools":{"serial":{"onboard":1},"cklink":{}}}}]
#print(w806)
# target_abs="C:/Users/debug/.stduino/packages/stdboards.json"
# fo = open(target_abs, mode='r', encoding='UTF-8')
# pio_boards = fo.read()
# fo.close()  # cmd_args["cmd_rtlib"]
# w806 = json.loads(pio_boards)
w806 =[]

#调用前先判断是否已经正常安装pio
class PioProjectManage():
    def __init__(self):
        if stdinit.platform_is == "Win":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio" + '"'

        elif stdinit.platform_is == "Linux":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'

        elif stdinit.platform_is == "Darwin":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'





        #self.projects_dir = stdinit.stdenv+"/Documents/Stduino/Projects"
        #

        pass



    def check_net(self):
        try:
            return stdinit.std_signal_gobal.is_connected()

        except:
            return False
    def project_boards(self):#待完善
        try:
            target = stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json"
            if stdinit.pio_boards == None:

                if self.check_net():
                    if os.path.exists(target):
                        target_ini = stdinit.stdenv + "/.stduino/packages/pioenv/stdpio.ini"
                        if os.path.exists(target_ini):
                            pass
                        else:
                            copy2(stdinit.abs_path + "/tool/packages/stdpio.ini",
                                  stdinit.stdenv + "/.stduino/packages/pioenv/stdpio.ini")

                        stdinit.pro_conf.read(target_ini, encoding="utf-8")  # python3
                        now_time = int(time.time())
                        update_time = int(stdinit.pro_conf.get('update', 'boards_update'))

                        if (now_time - update_time) > 150:#456400:
                            # print(223)

                            try:
                                stdinit.pro_conf.set('update', 'boards_update', str(now_time))
                                stdinit.pro_conf.write(open(target_ini, 'w'))
                                cmd = self.pio_env+" boards --json-output"

                                rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道

                                try:
                                    data = str(rest.stdout.read(), encoding='gbk')
                                except:
                                    data = rest.stdout.read()
                                rest.stdout.close()
                                # cmd = stdinit.stdenv+"/packages/stdenv/Scripts/python.exe "+stdinit.stdenv + "/packages/stdenv/Lib/site-packages/stduino/function/cores/commands/commands.py boards search --json-output"
                                # rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
                                #
                                # try:
                                #     data2 = str(rest.stdout.read(), encoding='gbk')
                                # except:
                                #     data2 = rest.stdout.read()
                                # rest.stdout.close()
                                data1=json.loads(data)
                                # data3 = json.loads(data2)

                                # print(type(data1))
                                # print(type(data3))
                                # print(data1)
                                # print(data3)
                                stdinit.pio_boards =data1 #+ data3  # 字符串转json


                                fo = open(target, "w", encoding="utf-8", newline="")  # gbk_utf_type
                                fo.write(json.dumps(stdinit.pio_boards))
                                fo.close()
                                #stdinit.pio_boards=stdinit.pio_boards+w806
                                #print(type(stdinit.pio_boards))
                                return True
                            except:

                                stdinit.std_signal_gobal.stdprintln()
                                return False
                        else:
                            try:

                                fo = open(target, mode='r', encoding='UTF-8')
                                stdinit.pio_boards = fo.read()
                                fo.close()  # cmd_args["cmd_rtlib"]
                                stdinit.pio_boards = json.loads(stdinit.pio_boards)

                                #stdinit.pio_boards=stdinit.pio_boards+w806

                                return True
                            except:

                                stdinit.std_signal_gobal.stdprintln()
                                return False
                    else:

                        #target_abs = stdinit.abs_path + "/tool/packages/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                        try:
                            cmd = self.pio_env + " boards --json-output"

                            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道

                            try:
                                data = str(rest.stdout.read(), encoding='gbk')
                            except:
                                data = rest.stdout.read()
                            rest.stdout.close()
                            cmd = stdinit.stdenv+"/packages/stdenv/Scripts/python.exe "+stdinit.stdenv + "/packages/stdenv/Lib/site-packages/stduino/function/cores/commands/commands.py boards search --json-output"
                            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道

                            try:
                                data2 = str(rest.stdout.read(), encoding='gbk')
                            except:
                                data2 = rest.stdout.read()
                            rest.stdout.close()
                            data1 = json.loads(data)
                            data3 = json.loads(data2)

                            # print(type(data1))
                            # print(type(data3))
                            # print(data3)
                            # return
                            stdinit.pio_boards = data1 + data3  # 字符串转json

                            fo = open(target, "w", encoding="utf-8", newline="")  # gbk_utf_type
                            fo.write(json.dumps(stdinit.pio_boards))
                            fo.close()
                            return True
                        except:
                            stdinit.std_signal_gobal.stdprintln()
                            return False
                else:

                    if os.path.exists(target):
                        try:
                            fo = open(target, mode='r', encoding='UTF-8')
                            stdinit.pio_boards = fo.read()
                            fo.close()  # cmd_args["cmd_rtlib"]
                            stdinit.pio_boards = json.loads(stdinit.pio_boards)
                            stdinit.pio_boards = stdinit.pio_boards
                            return True
                        except:
                            stdinit.std_signal_gobal.stdprintln()
                            return False
                    else:
                        # return False

                        target_abs = stdinit.abs_path + "/tool/packages/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                        try:
                            fo = open(target_abs, mode='r', encoding='UTF-8')
                            stdinit.pio_boards = fo.read()
                            fo.close()  # cmd_args["cmd_rtlib"]
                            stdinit.pio_boards = json.loads(stdinit.pio_boards)
                            stdinit.pio_boards = stdinit.pio_boards
                            copy2(target_abs,target)
                            return True
                        except:
                            stdinit.std_signal_gobal.stdprintln()
                            return False
            else:
                if os.path.exists(target):
                    return True
                else:
                    return False
        except:
            stdinit.std_signal_gobal.stdprintln()







            # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"




        # target = stdinit.stdenv + "/.stduino/packages/pioenv/stdpio.ini"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
        # if os.path.exists(target):
        #     pass
        #
        #     return True
        # else:
        #     return False
        # cmd = stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio boards --json-output"
        # rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
        # data = str(rest.stdout.read(), encoding='gbk')
        # rest.stdout.close()
        # projectboards = json.loads(data)  # 字符串转json
        # return projectboards


    def project_config(self):#待完善
        try:
            cmd = self.pio_env+" project config --json-output"
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道

            try:
                data = str(rest.stdout.read(), encoding='gbk')
            except:
                data = rest.stdout.read()
            rest.stdout.close()
            projectconfig = json.loads(data)  # 字符串转json
            return projectconfig
        except:
            stdinit.std_signal_gobal.stdprintln()


    #Recently updated
    #Recently added
    #Recent keywords
    #Popular keywords
    # Featured: Today
    # Featured: Week
    # Featured: Month
    def project_data(self):
        try:
            cmd = self.pio_env+" project data --json-output"
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            data = str(rest.stdout.read(), encoding='gbk')
            rest.stdout.close()
            libs = json.loads(data)  # 字符串转json
            return libs
        except:
            stdinit.std_signal_gobal.stdprintln()


        # print("data2['name']: ", libs[0]['ownername'])

    def generate_project_main(self,framework):
        try:
            time.sleep(0.5)
            stdinit.std_signal_gobal.std_work_space()
            main_content = None
            if framework == "arduino":
                main_content = "\n".join(
                    [
                        "#include <Arduino.h>",
                        "",
                        "void setup() {",
                        "  // put your setup code here, to run once:",
                        "}",
                        "",
                        "void loop() {",
                        "  // put your main code here, to run repeatedly:",
                        "}",
                        "",
                    ]
                )
            elif framework == "mbed":
                main_content = "\n".join(
                    [
                        "#include <mbed.h>",
                        "",
                        "int main() {",
                        "",
                        "  // put your setup code here, to run once:",
                        "",
                        "  while(1) {",
                        "    // put your main code here, to run repeatedly:",
                        "  }",
                        "}",
                        "",
                    ]
                )
            if not main_content:
                return True
            src_dir = stdinit.projects_dir + stdinit.project_name + "/src"
            main_path = stdinit.projects_dir + stdinit.project_name + "/src/main.cpp"
            if os.path.isfile(main_path):
                return True
            if not os.path.isdir(src_dir):
                os.makedirs(src_dir)

            with open(main_path, "w") as fp:
                fp.write(main_content.strip())
            return True
        except:
            stdinit.std_signal_gobal.stdprintln()


    def project_std_init(self,upload_m,framework):
        target_path = stdinit.projects_dir + stdinit.project_name
        cmd = stdinit.stdenv+"/packages/stdenv/Scripts/python.exe "+stdinit.stdenv + "/packages/stdenv/Lib/site-packages/stduino/function/cores/commands/commands.py"
        if upload_m == "Disable":  # upload_m 调试前根据这个进行判断是否可以支持调试
            cmd = cmd + " project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework
        else:
            cmd = cmd + " project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework + " -O debug_tool=" + upload_m
        try:
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)
                stdinit.std_signal_gobal.std_echo_msg(1,s1)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
            proc.stdout.close()
            stdinit.std_signal_gobal.std_work_space()
            return True
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False



    def project_init(self,upload_m,framework):


        try:
            target_path = stdinit.projects_dir + stdinit.project_name
          #大小写区分待解决 win下
            if os.path.exists(target_path):
                pass
            else:
                os.makedirs(target_path)
            stdinit.std_signal_gobal.std_project_path_observer(target_path)
            stdinit.std_signal_gobal.std_main_ui_change(1)
            target_path='"' + target_path+ '"'


            if upload_m=="Disable":#upload_m 调试前根据这个进行判断是否可以支持调试
                cmd = self.pio_env+" project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework
            else:
                cmd = self.pio_env+" project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework + " -O debug_tool=" + upload_m


            # if upload_m=="Disable":#upload_m 调试前根据这个进行判断是否可以支持调试
            #     cmd = stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework
            # else:
            #     cmd = stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework + " -O debug_tool=" + upload_m
            #

            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1=str(line)
                #print(s1)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
            proc.stdout.close()


            return self.generate_project_main(framework)
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False

    def project_conf_read(self):#待完善其他目录时情况
        try:
            if stdinit.pro_conf == None:
                pass
            else:
                stdinit.pro_conf.clear()
            stdinit.pro_conf.read(stdinit.ini_path, encoding="utf-8")  # python3
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False

        #print(cfgpath)  # cfg.ini的路径
        # 创建管理对象
        # 读ini文件
        #stdinit.ini_path = stdinit.projects_dir + stdinit.project_name + "/platformio.ini"

    def project_conf_save(self):
        try:
            stdinit.pro_conf.write(open(stdinit.ini_path, 'w'))
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False



        # 获取所有的section
        #
        #conf.write(open(cfgpath, "w", encoding="utf-8"))  # r+模式


    def project_conf_add(self,  section,item,value):
        try:
            self.project_conf_read()
            # # 添加一个section
            sections = stdinit.pro_conf.sections()
            value_s = 0
            for sec in sections:
                if sec == section:
                    value_s = 1
                pass
            if value_s == 1:
                stdinit.pro_conf.set(section, item, str(value))

            else:
                stdinit.pro_conf.add_section(section)
                stdinit.pro_conf.set(section, item, str(value))
                # 往select添加key和value
                pass

            stdinit.project_conf_save()
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False

    def project_conf_del(self, section,item,si_value):#0删item 1删section
        try:
            self.project_conf_read()
            if si_value == 0:
                # 删除一个 section中的一个 item（以键值KEY为标识）
                stdinit.pro_conf.remove_option(section, item)
            else:
                # 删除整个section这一项
                stdinit.pro_conf.remove_section(section)
                pass
            self.project_conf_save()
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False

    def project_conf_update(self, section,item,value):
        try:
            self.project_conf_read()
            # 往select添加key和value 如果已存在即为修改
            stdinit.pro_conf.set(section, item, value)
            self.project_conf_save()
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False

    def project_conf_find(self, board, framework):
        try:
            self.project_conf_read()
            sections = stdinit.pro_conf.sections()
            #print(sections)  # 返回list
            items = stdinit.pro_conf.items(sections[0])
            #print(items)  # list里面对象是元祖
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False






# import sys
from PyQt5.QtWidgets import QWidget, QComboBox, QLabel,QPushButton,QGridLayout
from PyQt5.QtCore import Qt
import threading
from .stdedit import stdinit
class StdMakeBoard(QWidget):
    def __init__(self):
        super().__init__()
        try:
            # 设置标题
            # self.setWindowTitle('ComBox例子')
            # # 设置初始界面大小
            # self.resize(300, 200)
            self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
            # self.setWindowIcon(QIcon("appearance/img/st.PNG"))
            self.setStyleSheet('background: DimGrey')

            # 实例化QComBox对象

            self.cb = QComboBox(self)
            self.make_sec=1
            # self.cb.move(100, 20)
            self.tip_text = QLabel("该示例中含有多个芯片型号\n\n请选择目标芯片型号")

            # 单个添加条目
            # self.cb.addItem('C')
            # self.cb.addItem('C++')
            # self.cb.addItem('Python')
            # 多个添加条目

            # wlayout = QVBoxLayout()

            # 局部布局：水平，垂直，网格，表单
            glayout = QGridLayout()
            # line edit
            # LineEdit1 = QLineEdit()
            sec_board = QPushButton("确定")
            sec_board.clicked.connect(self.print_value)
            no_board = QPushButton("取消")
            no_board.clicked.connect(self.cancel)

            glayout.addWidget(self.tip_text, 1, 0)  # name platform board fromwork  下载方式
            glayout.addWidget(self.cb, 2, 0)

            glayout.addWidget(sec_board, 11, 0)
            glayout.addWidget(no_board, 11, 1)
            # 准备四个控件
            # gwg = QWidget()
            # # 使用四个控件设置局部布局
            # gwg.setLayout(glayout)
            # # 将四个控件添加到全局布局中
            # wlayout.addWidget(gwg)
            self.setLayout(glayout)
        except:
            stdinit.std_signal_gobal.stdprintln()
        #self.init_cores()

        # 信号
        # self.cb.currentIndexChanged[str].connect(self.print_value) # 条目发生改变，发射信号，传递条目内容
        # self.cb.currentIndexChanged[int].connect(self.print_value)  # 条目发生改变，发射信号，传递条目索引
        # self.cb.highlighted[str].connect(self.print_value)  # 在下拉列表中，鼠标移动到某个条目时发出信号，传递条目内容
        # self.cb.highlighted[int].connect(self.print_value)  # 在下拉列表中，鼠标移动到某个条目时发出信号，传递条目索引
    # def init_cores(self):
    #     self.cb.addItems(['Java', 'C#', 'PHP'])

    def make_run(self):
        try:
            if self.make_sec==1:
                stdinit.Std_Make.std_run()
            elif self.make_sec==2:
                stdinit.Std_Make.std_upload()
            elif self.make_sec==3:
                stdinit.Std_Make.std_upload()
            elif self.make_sec==4:
                stdinit.Std_Make.std_clean()


        except:
            stdinit.std_signal_gobal.stdprintln()

    def print_value(self):
        try:
            stdinit.std_make_board = self.cb.currentText()
            t1 = threading.Thread(target=self.make_run, name='make_run')
            t1.setDaemon(True)
            t1.start()
            self.close()

        except:
            stdinit.std_signal_gobal.stdprintln()

    def cancel(self):
        try:

            self.close()
            stdinit.std_signal_gobal.std_make_signal(0)
        except:
            stdinit.std_signal_gobal.stdprintln()




# if __name__ == '__main__':
#     app = QApplication(sys.argv)
#     comboxDemo = StdMakeBoard()
#     comboxDemo.show()
#     sys.exit(app.exec_())


# -*- coding: utf-8 -*-
#import appearance.language.zh_hans

#import appearance.language.en

import os
import configparser
from platform import system
stdenv = None
if system() == "Windows" or system() == "cli":
    stdenv = os.environ["USERPROFILE"].replace('\\', '/')

elif system() == "Linux":
    stdenv = os.environ["HOME"].replace('\\', '/')
elif system() == "Darwin":
    stdenv = os.environ["HOME"].replace('\\', '/')
cf =configparser.ConfigParser()
config_p=stdenv+"/.stduino/session/config.cfg"
if os.path.exists(config_p):
    pass
else:
    config_p="./appearance/config.cfg"
cf.read(config_p)  # 读取配置文件内容

#此处读出的均为字符串

version_id=cf.get('default_c', 'version_id')
keywordversion_id=cf.get('default_c', 'keywordversion_id')
boardversion_id=cf.get('default_c', 'boardversion_id')
updateversion_id=cf.get('default_c', 'updateversion_id')
updatetime=cf.get('default_c', 'updatetime')

creatshortcut= cf.get('language', 'st1')  # 返回_a章节里面key为a_key2的值，返回为int类型

#test
#xyz=zh_hans.xyz

#img
#pathstaus2





# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""

from ..stdmsg import reso
from .stdlexer import QsciLexerC
from .. import commenter
import os
from ...conf import res
import re
import threading
from PyQt5.QtWidgets import QAction,QMenu,QShortcut
import subprocess
from PyQt5.QtCore import QFileInfo,Qt,QSize
from PyQt5.QtGui import QColor,QImage,QPixmap,QFont,QFontMetrics,QCursor,QKeySequence
from PyQt5.Qsci import QsciScintilla,QsciAPIs
from . import stdinit
from subprocess import run
from shutil import copy2
class StdTabEdit(QsciScintilla):
    NextId = 1
    def __init__(self, filename, parent=None):
        super(StdTabEdit, self).__init__(parent)
        try:
            self.setAttribute(Qt.WA_DeleteOnClose)
            # self.setAttribute(Qt.WA_InputMethodEnabled, True) 启动后为中文输入，待完善为英文输入

            self.filename = filename
            self.mark_list=[]
            self.margin_double=None
            self.jump_word = None
            #self.textChange_s=StdSinal()
            self.textChanged.connect(self.textChange) #移植至ui中绑定测试
            self.linesChanged.connect(self.line_change)
            #self.textEdit.textChanged.connect(self.editchange)
            self.setContextMenuPolicy(Qt.CustomContextMenu)  # 右键菜单，如果不设为CustomContextMenu,无法使用customContextMenuRequested
            self.customContextMenuRequested.connect(self.showContextMenu)
            # self.setWrapMode(QsciScintilla.WrapWord)  # 编辑区宽度设置   不设置会出现水平进度条

            self.setWrapIndentMode(QsciScintilla.WrapIndentFixed)
            self.setIndentationsUseTabs(False)  # 设置TAB键及竖格效果
            self.setTabWidth(1)
            self.setIndentationGuides(True)
            self.setTabIndents(True)  # 设置TAB键及竖格效果
            self.setAutoIndent(True)  # 开启自动缩进
            # self.setCaretLineVisible(True)  #显示选中的行号 目前没效果就没开启
            # self.setColor(QColor("#008000"), QsciLexerCPP.CommentDoc)  # 文档注释 /**开头的颜色
            # self.setColor(QColor("#008000"), QsciLexerCPP.Comment)  # 块注释 的颜色
            # self.setColor(QColor("#008000"), QsciLexerCPP.CommentLine)  # 行注释的颜色
            # self.setColor(QColor("#007f7f"), QsciLexerCPP.Number)  # 数字 的颜色
            # print(QsciLexerCPP.Number)
            # self.setColor(QColor("#ff00ff"), QsciLexerCPP.DoubleQuotedString)  # 双引号字符串的颜色
            # self.setColor(QColor("#ff00ff"), QsciLexerCPP.SingleQuotedString)  # 单引号字符的颜色
            # self.setColor(QColor("#be07ff"), QsciLexerCPP.PreProcessor)  # 预编译语句的颜色
            # self.setColor(QColor("#191970"), QsciLexerCPP.Operator)
            self.setCaretForegroundColor(QColor("#00E5EE"))  # 设置光标颜色#ff0000ff
            self.setCaretLineVisible(True)  # 设置光标所在行背景凸显
            self.setCaretLineBackgroundColor(QColor("#2F4F4F"))  # 设置光标所在行背景颜色
            #
            #self.setMarkerBackgroundColor(QColor("#191970"))
            # self.markerLine(5)
            # self.markersAtLine(6)
            self.setCaretWidth(20)  # 设置光标宽度
            self.setMarginType(0, QsciScintilla.NumberMargin)
            self.setMarginWidth(0,30)  # 设置行号显示及显示的宽度#0, "0000000" 40  1位 15
            self.setMarginType(1, QsciScintilla.SymbolMargin)
            self.setMarginWidth(1,40)#设置断点的占位宽度
            #self.setEolMode(QsciScintilla.EolWindows)
            #self.setEolVisibility(True)
            #self.setMaximumWidth(50)
            self.setFolding(QsciScintilla.BoxedTreeFoldStyle)  # 折叠样式
            # self.setMarginsBackgroundColor(Qt.black)
            # self.setMarginBackgroundColor(QColor("#1fff"))
            self.setMarginsForegroundColor(QColor(121, 122, 123))  # 行号字体背景颜色
            # self.setFolding(QsciScintilla.BoxedFoldStyle)
            self.setFoldMarginColors(QColor(30, 30, 30), QColor(121, 122, 123))  # 折叠栏颜色  # 折叠背景颜色
            #self.setMarkerBackgroundColor(QColor(30, 30, 30))#


            self.setMarginsBackgroundColor(QColor(30, 30, 30))  # 行号背景颜色
            #editor.setMarginsBackgroundColor(PyQt5.QtGui.QColor(0x8a, 0x00, 0x34, 80))
            # Set the foreground color (the text on margins) of all margins.
            # If this is not set it defaults to Black.
            #self.setMarginsForegroundColor(QColor(0x00, 0x00, 0xff, 0xff))
            # Set the default font
            self.font = QFont()
            self.font.setFamily('Consolas')
            self.font.setFixedPitch(True)
            self.font.setPointSize(12)  # 行号字体大小
            self.setFont(self.font)
            # self.__editor.setMarginsFont(self.__editor.font)
            # Margin 0 is used for line numbers
            fontmetrics = QFontMetrics(self.font)
            self.setMarginsFont(self.font)
            # self.__editor.setMarginWidth(0, fontmetrics.width("000") + 6)
            self.setMarginLineNumbers(0, True)  # 是否显示行号
            self.setMarginSensitivity(1, True)



            # Indentation
            self.setIndentationWidth(4)
            self.setBackspaceUnindents(True)
            self.setEdgeMode(QsciScintilla.EdgeLine)
            self.setEdgeColumn(80)
            self.setEdgeColor(QColor(0, 0, 0))



            sym_0 = QImage("appearance/img/debug_stop_next.svg").scaled(QSize(26, 26))
            sym_1 = QImage("appearance/img/img_dot4.svg").scaled(QSize(20, 20))
            sym_2 = QImage("appearance/img/img_dot4.svg").scaled(QSize(20, 20))
            sym_3 = QImage("appearance/img/img_dot4.svg").scaled(QSize(20, 20))
            # sym_4 = QsciScintilla.Circle


            self.markerDefine(sym_0, 0)
            self.markerDefine(sym_1, 1)
            self.markerDefine(sym_2, 2)
            self.markerDefine(sym_3, 3)
            self.markerDefine(self.Background, 4)
            self.markerDefine(self.Underline, 5)
            self.setMarkerBackgroundColor(QColor("#191970"), 4)
            #self.setMarkerForegroundColor(QColor("#191970"), 4)
            self.setMarkerBackgroundColor(QColor("#696969"), 5)#000000  #2E8B57
            #self.setMarkerForegroundColor(QColor("#000000"), 5)

            self.setMarginMarkerMask(1, 0b1111)
            # self.setMarginMarkerMask(1, 0x02)

            # 6. Margin mouse clicks
            # -----------------------
            self.setMarginSensitivity(1, True)
            self.marginClicked.connect(self.__margin_left_clicked)

            # self.marginRightClicked.connect(self.__margin_right_clicked)





            # // 每输入1个字符就出现自动完成的提示
#####################################################################




            # Comment feature goes here

            # self.commenter.

            QShortcut(QKeySequence(Qt.ALT + Qt.Key_P), self, self.startsetcolor)
            QShortcut(QKeySequence(Qt.ALT + Qt.Key_F), self, self.Format)
            QShortcut(QKeySequence(Qt.CTRL + Qt.Key_F), self, self.Find)
            self.commenter = commenter.Commenter(self, "//")
            stdinit.std_signal_gobal.comment_uncomment.connect(self.commenter.toggle_comments)
            #QShortcut(QKeySequence(Qt.CTRL + Qt.Key_Slash), self, self.commenter.toggle_comments)
            # QtGui.QKeySequence(QtCore.Qt.ALT + QtCore.Qt.Key_E)

            lexer= QsciLexerC(self)


            self.api = QsciAPIs(lexer)

            self.setLexer(lexer)
            self.setAutoCompletionReplaceWord(False)#替换之后的文字

            self.setAutoCompletionSource(QsciScintilla.AcsAll)  # 自动补全。对于所有Ascii字符 AcsAPIs  AcsAll

            self.setAutoCompletionCaseSensitivity(True)  # 设置自动补全大小写敏感
            # self.setAutoCompletionFillupsEnabled(False)
            #

            self.SCN_AUTOCCOMPLETED.connect(self._correct_autocompletion)  #完成获取
            #self.SCN_AUTOCSELECTIONCHANGE(self._correct_autocompletion)

            # SendScintilla(self, int, wParam: int = 0, lParam: int = 0) -> int
            # SendScintilla(self, int, int, sip.voidptr) -> int
            # SendScintilla(self, int, int, bytes) -> int
            # SendScintilla(self, int, bytes) -> int
            # SendScintilla(self, int, bytes, bytes) -> int
            # SendScintilla(self, int, int) -> int
            # SendScintilla(self, int, int, int, bytes) -> int
            # SendScintilla(self, int, int, Union[QColor, Qt.GlobalColor]) -> int
            # SendScintilla(self, int, Union[QColor, Qt.GlobalColor]) -> int
            # SendScintilla(self, int, int, QPainter, QRect, int, int) -> int
            # SendScintilla(self, int, int, QPixmap) -> int
            # SendScintilla(self, int, int, QImage) -> int


            self.setAutoCompletionThreshold(1)
            self.setAutoCompletionUseSingle(QsciScintilla.AcusNever)  # AcusAlways AcusNever

            # // 设置缩进的显示方式
            #
            self.setIndentationGuides(QsciScintilla.SC_IV_LOOKBOTH)
            self.setBraceMatching(QsciScintilla.SloppyBraceMatch)

            self.setUtf8(True)


            # Sets whether the characters to the right of the autocompletion
            # will be overwritten when an autocompletion is selected.


            self.SendScintilla(QsciScintilla.SCI_SETSCROLLWIDTHTRACKING, 1)
            #self.SendScintilla(QsciScintilla.SCI_AUTOCSETIGNORECASE, 0)
            #self.setAutoCompletionFillupsEnabled(False)


            self.setCallTipsVisible(0)  # 同时显示多条提示
            self.setCallTipsPosition(QsciScintilla.CallTipsAboveText)
            self.setCallTipsStyle(QsciScintilla.CallTipsContext)  # 出现的时间

            # Multiple cursor support
            self.SendScintilla(QsciScintilla.SCI_SETMULTIPLESELECTION, True)
            self.SendScintilla(QsciScintilla.SCI_SETMULTIPASTE, 1)
            self.SendScintilla(QsciScintilla.SCI_SETADDITIONALSELECTIONTYPING, True)

            self.SendScintilla(QsciScintilla.SCI_SETCODEPAGE, QsciScintilla.SC_CP_UTF8)


            # Register an image that will be displayed with an autocompletion
            autocompletion_image = QPixmap("appearance/img/st.PNG")
            self.registerImage(1, autocompletion_image)
            # Create a list of autocompletions


            # Add the functions to the api
            # self.auto_key_load()
            t1 = threading.Thread(target=self.auto_key_load, name='auto_key_load')
            t1.setDaemon(True)
            t1.start()
            # key_load = threading.Thread(target=self.auto_key_load, name='key_load')
            # #key_load.setDaemon(True)
            # key_load.start()

            # Example of removing an entry

            # self.__editor.setText(str)


            if not self.filename:
                self.filename = "Unnamed_{0}".format(StdTabEdit.NextId)
                StdTabEdit.NextId += 1
            # self.setModified(False) #新建文件时首次设置无改动
            self.setWindowTitle(QFileInfo(self.filename).fileName())

            # -------------------------------- #
            #  Connect function to indicators  #
            # -------------------------------- #

            #self.indicatorClicked.connect(self.indicator_clicked)
            # self.SCN_HOTSPOTDOUBLECLICK.connect(self.tes)


        except:
             stdinit.std_signal_gobal.stdprintln()



    # def indicator_clicked(self, line, index, keys):
    #     print(line)
    #     print(index)
    #     pos = self.positionFromLineIndex(line, index)
    #
    #     start = self.SendScintilla(QsciScintilla.SCI_INDICATORSTART, 0, pos)
    #     end = self.SendScintilla(QsciScintilla.SCI_INDICATOREND, 0, pos)
    #     text = self.text()[start:end]
    #     print("indicator '{}' clicked in line '{}', index '{}'".format(text, line, index))
    #
    #     relPath, line = self.__lexer.parser.where_to_jump(text)
    #     linefocus = int(line) - 1
    #     print(linefocus)
    #     print("jump to file: " + relPath)
    #
        # relPath = globals.projectFolderPath + '\\' + relPath
        #QTimer.singleShot(100, functools.partial(self.indicator_clicked_delayed, line, index, keys))

    ''''''


    def auto_key_load(self):
        try:
            if stdinit.auto_key_l == 0:

                stdinit.auto_key_l = 1

                # autocompletions = [
                #     "int",
                #     "#include",
                #     "#defind",
                #     "goto",
                #     "sizeof",
                #     "static",
                #     "const",
                #     "pow",
                #     "sqrt",
                #     "min",
                #     "max",
                #     "abs",
                #     "constrain",
                #     "long",
                #     "map",
                #     "if",
                #     "else",
                #     "while",
                #     "for",
                #     "switch",
                #     "case",
                #     "do",
                #     "break",
                #     "continue",
                #     "return",
                #     "void",
                #     "double",
                #     "float",
                #     "boolean",
                #     "String",
                #     "char",
                #     "unsigned",
                #     "HIGH",
                #     "LOW",
                #     "INPUT",
                #     "OUTPUT",  # This call tip has a description and an image
                #     "INPUT_PULLUP",
                #     "INPUT_PULSE",
                #     "INPUT_ANALOG",
                #     "OUTPUT_PULSE",
                #     "INPUT_PULLDW",
                #     "peek",
                #     "peekString",
                #     "delay",
                #     "delay(ms)",
                #     "delayMicroseconds",
                #     "delayMicroseconds(us)",
                #     "pinMode",
                #     "pinMode(pin, INPUT )//浮空输入",
                #     "pinMode(pin, OUTPUT)//推挽输出",
                #     "pinMode(pin, INPUT_PULLUP)//上拉输入",
                #     "digitalWrite",  # This call tip has a description and an image
                #     "digitalWrite(pin, value)",
                #     "digitalRead",
                #     "digitalRead(pin)",
                #     "analogRead",
                #     "analogRead(pin)",
                #     "analogWrite",
                #     "analogWrite(pin,value)",
                #     "millis",
                #     "micros",
                #     "random",
                #     "map",
                #     "tone",
                #     "noTone",
                #     "Serial.begin",
                #     "Serial1.begin",
                #     "Serial2.begin",
                #     "Serial.print",
                #     "Serial1.print",
                #     "Serial2.print",
                #     "Serial.println",
                #     "Serial1.println",
                #     "Serial2.println",
                #     "Serial.read",
                #     "Serial1.read",
                #     "Serial2.read",
                #     "Serial.available",
                #     "Serial1.available",
                #     "Serial2.available"
                # ]
                # for ac in autocompletions:
                #     self.api.add(ac)
                try:
                    pass
                    # for root, dirs, files in os.walk(res.library):
                    #     for name in files:
                    #
                    #         if name == 'keywords.txt':
                    #             try:
                    #                 stdinit.auto_key_list.append(os.path.join(root, name))
                    #                 #self.api.load(os.path.join(root, name))
                    #
                    #             except:
                    #                 pass
                    # # for root, dirs, files in os.walk(res.library, topdown=False):
                    # pass

                except:
                    stdinit.std_signal_gobal.stdprintln()

            try:

                ke_len = len(stdinit.auto_key_list)
                for i in range(ke_len):
                    self.api.load(stdinit.auto_key_list[i])
                target = stdinit.stdenv + "/.stduino/session/keywords.txt"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                if os.path.exists(target):
                    self.api.load(target)
                else:
                    target2 = stdinit.stdenv + "/.stduino/session"
                    if os.path.exists(target2):
                        copy2('./appearance/stduino_json/keywords.txt',target)
                        self.api.load(target)
                    else:
                        os.makedirs(target2)
                        copy2('./appearance/stduino_json/keywords.txt', target)
                        self.api.load(target)


                # self.api.remove("entry_that_will_be_removed_later")
                # Prepare the QsciAPIs instance information
                self.api.prepare()
            except:
                stdinit.std_signal_gobal.stdprintln()
        except:
            stdinit.std_signal_gobal.stdprintln()








    splitter = re.compile(r"(\{\.|\.\}|\#|\'|\"\"\"|\n|\s+|\w+|\W)")


    def _correct_autocompletion(self, *args):

        print(args)
        try:
            word, from_index, to_index, length = args
            word = word.decode("utf-8")
            #print(2)
            current_line = self.getCursorPosition()[0] + 1
            #print(4)
            #self.line_list[current_line]

            line = "dsd"
            #print(1)
            for token in self.splitter.findall(self.text()):


                if token.lower() == word.lower():
                    #print(32)
                    self.setCursorPosition(current_line - 1, len("sdsdsdsd"))

                    # self.line_list[current_line] = token.join(line.rsplit(word, 1))
                    # self.setCursorPosition(current_line - 1, len(self.line_list[current_line]))
                    break

        except:
            stdinit.std_signal_gobal.stdprintln()
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容


    def __margin_left_clicked(self, margin_nr, line_nr, state):
        #print(margin_nr
        try:
            if self.margin_double == None:
                self.margin_double = 1
            else:
                self.margin_double = None
            # print("Margin clicked (left mouse btn)!")
            # print(" -> margin_nr: " + str(margin_nr))
            # print(" -> line_nr:   " + str(line_nr))
            # (xyxy)
            # 2,0    2,2
            #print(self.lineLength(line_nr))

            self.setSelection(line_nr,0,line_nr, self.lineLength(line_nr)-2)#遇到有中文的行时会多选
            text=self.selectedText().replace(" ","")
            self.setCursorPosition(line_nr,0)
            if len(text)>0:
                if text[0].encode('UTF-8').isalpha():
                    if self in stdinit.mark_object:
                        pass
                    else:
                        stdinit.mark_object.append(self)
                    mark_line=self.filename + ":" + str(line_nr + 1)
                    if self.markersAtLine(line_nr) > 1:
                        if stdinit.debug_mark_start == 1:
                            stdinit.std_signal_gobal.std_gdb_cmd("d " + mark_line)
                            self.markerDelete(line_nr,1)
                            self.markerDelete(line_nr, 5)
                            #self.mark_list.remove(mark_line)
                            #stdinit.manual_mark.remove(mark_line)
                        else:
                            self.markerDelete(line_nr, 1)
                            self.markerDelete(line_nr, 5)

                            #stdinit.manual_mark.remove(mark_line)
                    else:
                        if stdinit.debug_mark_start == 1:
                            stdinit.std_signal_gobal.std_gdb_cmd("break " + mark_line)
                            # stdinit.manual_mark.append(mark_line)
                            a = self.markerAdd(line_nr, 1)
                            #self.markerAdd(line_nr, 4)
                            self.markerAdd(line_nr, 5)

                            self.mark_list.append(a)

                            # print(a)
                            # print(12)
                        else:
                            # stdinit.manual_mark.append(mark_line)
                            a = self.markerAdd(line_nr, 1)
                            self.markerAdd(line_nr,5)
                            #self.markerAdd(line_nr, 4)
                            #self.markerDelete(line_nr, 5)
                            #print(22222222222222)
                            self.mark_list.append(a)
                            # self.markerAdd(line_nr, 0)
                            # self.markerDelete(line_nr, margin_nr)
                            # a1 = self.markersAtLine(line_nr)

                            # print(a1)
                            #self.markerDeleteHandle(1)

                            #print(self.markerLine(a))

                            # a=self.markerDeleteHandle(1)

                            # print(a)
                            # print(12222222222)




        except:
            stdinit.std_signal_gobal.stdprintln()


    ''''''

    def __margin_right_clicked(self, margin_nr, line_nr, state):
        try:
            print("Margin clicked (right mouse btn)!")
            print(" -> margin_nr: " + str(margin_nr))
            print(" -> line_nr:   " + str(line_nr))
            print("")
        except:
            stdinit.std_signal_gobal.stdprintln()






    # def mouseDoubleClickEvent(self, args):
    #     try:
    #
    #         if self.margin_double==1:
    #             QsciScintilla.mouseDoubleClickEvent(self, args)
    #             pass
    #         else:
    #             text = self.wordAtPoint(args.pos())#temp_text=text.replace("_","")
    #             #print(text)
    #             if text == "":  # temp_text.encode( 'UTF-8' ).isalpha():
    #                 QsciScintilla.mouseDoubleClickEvent(self, args)
    #                 pass#temp_text.encode( 'UTF-8' ).isalpha()
    #             else:
    #                 relPath, line = stdinit.goto_init.where_to_jump(text)
    #                 if relPath == "0":
    #                     stdinit.std_signal_gobal.std_echo_msg(1, "未找到相关声明:" + text)
    #                     QsciScintilla.mouseDoubleClickEvent(self, args)
    #                 else:
    #                     linefocus = int(line) - 1
    #                     stdinit.std_signal_gobal.std_jump_to_look(relPath, linefocus, text)
    #     except:
    #         stdinit.std_signal_gobal.stdprintln()


    def keyPressEvent(self, e):
        try:
            # event->key() == Qt::Key_Return & & (event->modifiers() & Qt::ControlModifier)
            if (e.key() == Qt.Key_Slash) and (e.modifiers() == Qt.ControlModifier):

                self.commenter.toggle_comments()
            if res.std_auto==1:


                    # self.setText('111212↑')
                    # res.std_auto
                if e.key() == 40:

                    if self.hasSelectedText():
                        self.replaceSelectedText("(" + self.selectedText() + ")")  # self.selectedText()

                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] - 1)
                    else:
                        self.insert("()")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] + 1)
                elif e.key() == 34:
                    if self.hasSelectedText():
                        self.replaceSelectedText("\"" + self.selectedText() + "\"")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] - 1)
                    else:
                        self.insert("\"\"")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] + 1)
                elif e.key() == 39:

                    if self.hasSelectedText():
                        self.replaceSelectedText("\'" + self.selectedText() + "\'")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] - 1)
                    else:
                        self.insert("\'\'")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] + 1)
                elif e.key() == 123:
                    if self.hasSelectedText():
                        self.replaceSelectedText("{" + self.selectedText() + "}")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] - 1)
                    else:
                        self.insert("{}")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] + 1)
                elif e.key() == 91:
                    if self.hasSelectedText():
                        self.replaceSelectedText("[" + self.selectedText() + "]")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] - 1)
                    else:
                        self.insert("[]")
                        cursor = self.getCursorPosition()
                        self.setCursorPosition(cursor[0], cursor[1] + 1)
                else:

                    QsciScintilla.keyPressEvent(self, e)

            else:

                QsciScintilla.keyPressEvent(self, e)
        except:
            stdinit.std_signal_gobal.stdprintln()

    #
    # def mousePressEvent(self,  e):
    #
    #     print(21)
    #
    #     QsciScintilla.mousePressEvent(self, e)

        #print(a)
    def mousePressEvent(self, event):
        try:
            if event.buttons() == Qt.RightButton:  # 右键按下
                self.jump_word = self.wordAtPoint(event.pos())
                # line = self.lineAt(event.pos())
            QsciScintilla.mousePressEvent(self, event)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def jump(self):
        try:
            relPath, line = stdinit.goto_init.where_to_jump(self.jump_word)
            if relPath == "0":
                stdinit.std_signal_gobal.std_echo_msg(1,"未找到相关声明"+self.jump_word)
            else:
                linefocus = int(line) - 1
                stdinit.std_signal_gobal.std_jump_to_look(relPath, linefocus, self.jump_word)

        except:
            stdinit.std_signal_gobal.stdprintln()

    def open_curflod(self):
        try:

            filename=self.filename
            title=self.windowTitle()
            filepath=filename.replace(title,"")


            if os.path.exists(filepath):
                if stdinit.platform_is == "Win":
                    os.startfile(filepath)

                elif stdinit.platform_is == "Linux":

                    os.system('xdg-open "%s"' % filepath)



                elif stdinit.platform_is == "Darwin":
                    subprocess.call(["open", filepath])

            else:
               # print("该文件路径有误！")
                stdinit.std_signal_gobal.std_echo_msg(0,"该文件路径有误！")

            pass

        except:
            stdinit.std_signal_gobal.stdprintln()


    def showContextMenu(self):  # 创建右键菜单
        try:

            #QsciScintilla.keyPressEvent(self, e)
            openflod_ac = QAction("打开当前文件所在文件夹", self)
            openflod_ac.setStatusTip("打开当前文件所在文件夹")
            openflod_ac.triggered.connect(self.open_curflod)
            jump_ac = QAction("转到声明" , self)
            jump_ac.setStatusTip("转到声明")
            jump_ac.triggered.connect(self.jump)
            paste_ac = QAction(reso.paste + ' Ctrl+V', self)
            paste_ac.setStatusTip(reso.paste)
            paste_ac.triggered.connect(self.paste)
            find_ac = QAction(reso.search + ' Ctrl+F', self)
            # find_ac.setShortcut('Ctrl+F')
            find_ac.setStatusTip(reso.search)
            find_ac.triggered.connect(self.Find)

            format_ac = QAction(reso.format1 + ' Alt+F', self)
            format_ac.setStatusTip(reso.format1)
            format_ac.triggered.connect(self.Format)

            comment_ac = QAction(reso.commenter + ' Ctrl+/', self)
            comment_ac.setStatusTip(reso.commenter)
            comment_ac.triggered.connect(self.commenter.toggle_comments)

            page_set = QAction(reso.editstyle + ' Alt+P', self)
            # page_set.setShortcut(QKeySequence(Qt.ALT+ Qt.Key_P))
            page_set.setStatusTip(reso.editstyle)
            page_set.triggered.connect(self.startsetcolor)

            # push_git = QAction('gitee_push', self)
            # push_git.setShortcut('')
            # push_git.setStatusTip('gitee_push')
            # push_git.triggered.connect(self.git_push)
            popMenu = QMenu()
            #temp_text = self.jump_word.replace("_", "")


            popMenu.addAction(openflod_ac)
            popMenu.addAction(find_ac)
            if stdinit.platform_is == "Win":
                if self.jump_word == "":  # temp_text.encode( 'UTF-8' ).isalpha():
                    pass
                else:
                    popMenu.addAction(jump_ac)
                popMenu.addAction(format_ac)

            elif stdinit.platform_is == "Linux":
                pass


            elif stdinit.platform_is == "Darwin":
                pass

            popMenu.addAction(comment_ac)
            popMenu.addAction(page_set)
            # popMenu.addAction(copy_ac)
            # popMenu.addAction(cut_ac)
            popMenu.addAction(paste_ac)
            # popMenu.addAction(select_ac)
            # popMenu.addAction(push_git)

            popMenu.exec_(QCursor.pos())
        except:
            stdinit.std_signal_gobal.stdprintln()




    # def git_push(self):
    #     pass
    #     # name = getpass.getuser()
    #
    #     if res.git_email == '' or res.git_project_url == '' or res.git_staus == '0':
    #         print('none git anme')
    #         if res.git_email == '':
    #             import getpass
    #             name = getpass.getuser()
    #             value, ok = QInputDialog.getText(self, "笔名", "请输入笔名:", QLineEdit.Normal, name)
    #             if ok == 1:
    #                 subprocess.call(['git', 'config', '--global', 'user.name', value])
    #             value, ok = QInputDialog.getText(self, "邮箱", "请输入联系邮箱:", QLineEdit.Normal, "123@qq.com")
    #             if ok == 1:
    #                 subprocess.call(['git', 'config', '--global', 'user.email', value])
    #                 setup.git_email(value)
    #             pass
    #         if res.git_project_url == '':
    #             value, ok = QInputDialog.getText(self, "Gitee仓库地址", "请输入正确的代码仓库地址:", QLineEdit.Normal,
    #                                              "https://gitee.com/stduino/stduino_projects.git")
    #             if ok == 1:
    #                 pos = os.path.abspath('.')
    #                 Path = r'C:\Users\sujinqiang\Documents\Stduino\git2'
    #                 os.chdir(Path)
    #                 setup.git_project_url(value)
    #                 subprocess.call(['git', 'remote', 'add', 'cs3', value])
    #                 os.chdir(pos)
    #             pass
    #         if res.git_staus == '0':
    #             value, ok = QInputDialog.getText(self, "Gitee仓库地址", "请输入正确的代码仓库地址:", QLineEdit.Normal,
    #                                              "https://gitee.com/stduino/stduino_projects.git")
    #             if ok == 1:
    #                 pos = os.path.abspath('.')
    #                 Path = r'C:\Users\sujinqiang\Documents\Stduino\git2'
    #                 os.chdir(Path)
    #                 setup.git_project_url(value)
    #                 if '.git' not in os.listdir():
    #                     subprocess.call(['git', 'clone', value])
    #                 # subprocess.call(['git', 'remote', 'add', 'cs3', value])
    #
    #                 os.chdir(pos)
    #             setup.git_staus("1")
    #     else:
    #         Path3 = r'C:\Users\sujinqiang\Documents\Stduino\git3'  # 本地git库路径
    #
    #         # change working directory
    #         pos = os.path.abspath('.')
    #         os.chdir(Path3)
    #         if '.git' not in os.listdir():
    #             git_remotes = subprocess.check_output(
    #                 ['git', 'clone', "https://gitee.com/stduino/stduino_projects.git", Path3])
    #             # new_repo = git.Repo.clone_from(url='https://gitee.com/stduino/stduino_projects.git', to_path=Path3)
    #             # git_remotes = subprocess.check_output(['git', 'config', 'user.name'])
    #             # c=subprocess.check_call(['git', 'init'])
    #             git_remotes_str = bytes.decode(git_remotes).strip()
    #             # print(git_remotes_str)
    #             # print(11)
    #
    #         subprocess.call(['git', 'add', "."])
    #         git_remotes = subprocess.check_output(['git', 'status'])
    #         git_remotes_str = bytes.decode(git_remotes).strip()
    #         a = re.search('Changes', git_remotes_str, re.I)
    #         print(git_remotes_str)
    #         if a == None:
    #             print("nothing to update")
    #             pass
    #         else:
    #             value, ok = QInputDialog.getText(self, "程序上传至仓库", "请输入本次更新的内容:", QLineEdit.Normal, "增加了驱动器")
    #             if ok == 1:
    #                 subprocess.call(['git', 'commit', '-m', value])
    #                 subprocess.check_call(['git', 'push', 'origin', 'master:master'])
    #             pass
    #         print('git')
    #         os.chdir(pos)
    #         pass

    def Find(self):
        try:
            pass
            stdinit.Std_Find.show()#待完善2021111
        except:
            stdinit.std_signal_gobal.stdprintln()


    def startsetcolor(self):  # startsetcolor

        # self.__lexer.setColor(QColor("#66CDAA"), QsciLexerCPP.CommentLine)  # 注释 #66CDAA  #1C1C1C  黑
        try:
            pass
            stdinit.Set_Back_Color1.show()#2021111注释
        except:
            stdinit.std_signal_gobal.stdprintln()

    def Format(self):
        try:

            file_path = self.SaveFile_format()
            options = "-xe"
            exe = '.\\appearance\AStyle\\bin\AStyle.exe'
            astyle = [exe, options, file_path]
            # retval = subprocess.call(astyle)#os.popen(
            retval = run(astyle, shell=True)
            # retval = os.popen(astyle)
            # print(retval)
            fo = open(file_path, mode='r', encoding='UTF-8')
            st = fo.read()
            fo.close()
            self.selectAll(True)
            # self.clear()
            self.replaceSelectedText(st)
            # self.insertAt('#include"stduino.h"\r\n',0,0)
            if retval:
                # print("Bad astyle return: " + str(retval))
                pass
                # error("Bad astyle return: " + str(retval))
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()


    def SaveFile_format(self):
        # global fileName
        try:

            pos = os.path.abspath('.')
            filename = pos + "\\appearance\AStyle\\bin\style.ino"
            fo = open(filename, "w", encoding="UTF-8", newline="")
            strr = self.text()
            # strr = self.__editor.text()
            # 在文件末尾写入一行self.combo.currentText()
            fo.write(strr)
            # 关闭文件
            fo.close()
            os.chdir(pos)
            return filename
        except:
            stdinit.std_signal_gobal.stdprintln()

    def save(self):
        try:

            if os.path.exists(self.filename):
                try:
                    fo = open(self.filename, "w", encoding="utf-8", newline="")  # gbk_utf_type
                    cs = self.text()
                    fo.write(cs)
                    fo.close()
                    self.setModified(False)
                    return 1
                except:
                    stdinit.std_signal_gobal.stdprintln()

            #the_window.setWindowTitle("Stduino IDE " + version_c + " [" + self.filename + ']')  # 文件名设置



        except:
            stdinit.std_signal_gobal.stdprintln()



    def load(self):
        try:

            fh = None
            try:
                # fh = QFile(self.filename)
                # if not fh.open(QIODevice.ReadOnly):
                #     fh.close()
                #     return 1
                #
                # stream = QTextStream(fh)
                fh = open(file=self.filename, mode='rb')  # 以二进制模式读取文件
                data = fh.read()  # 获取文件内容
                fh.close()  # 关闭文件unicode
                self.setText(data.decode("utf-8", 'ignore'))
                # self.insert(data.decode("utf-8", 'ignore'))
                self.setModified(False)

                if fh is not None:
                    fh.close()
                return 0
            except:
                stdinit.std_signal_gobal.stdprintln()
                fh.close()
                return 1
        except:
            stdinit.std_signal_gobal.stdprintln()



    # def keyPressEvent1(self,e):
    #
    #
    #     print(e.text())
    #     self.insert("\"")
    #     e.accept()
    # self.update()
    # QWidget.keyPressEvent(self, e)
    def line_change(self): #40
        try:
            m_lines = self.lines()
            margin_width = self.marginWidth(0)

            if m_lines < 10:
                m_lines = 30
                if m_lines == margin_width:
                    pass
                else:
                    self.setMarginWidth(0, m_lines)

            elif m_lines < 100:
                m_lines = 40
                if m_lines == margin_width:
                    pass
                else:

                    self.setMarginWidth(0, m_lines)
            elif m_lines < 1000:
                m_lines = 50
                if m_lines == margin_width:
                    pass
                else:
                    self.setMarginWidth(0, m_lines)

                pass
            elif m_lines < 10000:
                m_lines = 65
                if m_lines == margin_width:
                    pass
                else:
                    self.setMarginWidth(0, m_lines)

                pass
            elif m_lines < 10000:
                m_lines = 75
                if m_lines == margin_width:
                    pass
                else:
                    self.setMarginWidth(0, m_lines)
            else:
                m_lines = 80
                if m_lines == margin_width:
                    pass
                else:
                    self.setMarginWidth(0, m_lines)
        except:
            stdinit.std_signal_gobal.stdprintln()



        # print(m_lines)
        # m_lines=m_lines*15+35 # x=35+15*lines/10   0-9 10-99  100-999  1000-9999 10000-50000
        # margin_width=self.marginWidth(0)
        # print(margin_width)
        # if (margin_width<m_lines):
        #     self.setMarginWidth(0, m_lines)
        #     pass




        #self.setMarginWidth(0, 80)  # 设置行号显示及显示的宽度#0, "0000000"
    def textChange(self):

        try:
            # 待完善
            stdinit.savefileAction_staus =self.isModified()
            stdinit.saveallAction_staus = self.isModified()
            stdinit.forwardAction_staus = self.isRedoAvailable()
            stdinit.backAction_staus = self.isUndoAvailable()
            stdinit.std_signal_gobal.std_text_change()


            # self.savefileAction.setDisabled(not self.isModified())
            #
            # self.saveallAction.setDisabled(not self.isModified())
            # self.forwardAction.setDisabled(not self.isRedoAvailable())
            # self.backAction.setDisabled(not self.isUndoAvailable())stdtabedit

        except:
            stdinit.std_signal_gobal.stdprintln()







# -*- coding: utf-8 -*-

"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
import subprocess
import json
from ..cores.stdedit import stdinit
#from function.cores.stdedit import stdinit
#from function.cores.stdmsg import reso


#调用前先判断是否已经正常安装pio
class PioLibInstall():
    def __init__(self):
        if stdinit.platform_is == "Win":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio" + '"'

        elif stdinit.platform_is == "Linux":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'

        elif stdinit.platform_is == "Darwin":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'


    def lib_builtin(self):#待完善
        try:
            cmd = self.pio_env+" lib builtin --json-output"
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            try:
                data = str(rest.stdout.read(), encoding='gbk')
            except:
                data = str(rest.stdout.read())
            rest.stdout.close()
            libs = json.loads(data)  # 字符串转json
            return libs
        except:
            stdinit.std_signal_gobal.stdprintln()
    def check_net(self):
        try:
            return stdinit.std_signal_gobal.is_connected()

        except:
            return False


    #Recently updated
    #Recently added
    #Recent keywords
    #Popular keywords
    # Featured: Today
    # Featured: Week
    # Featured: Month
    def lib_stats(self):
        try:

            cmd = self.pio_env+" lib stats --json-output"
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            try:
                data = str(rest.stdout.read(), encoding='gbk')
            except:
                data = str(rest.stdout.read())
            rest.stdout.close()
            libs = json.loads(data)  # 字符串转json
            return libs
        except:
            stdinit.std_signal_gobal.stdprintln()


        # print("data2['name']: ", libs[0]['ownername'])
    def lib_search_k(self,k,p):#page
        try:

            cmd = self.pio_env+" lib search " + k + " --json-output --page " + str(p)
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            try:
                data = str(rest.stdout.read(), encoding='gbk')
            except:
                data = str(rest.stdout.read())

            rest.stdout.close()
            libs = json.loads(data)  # 字符串转json
            # print(libs['items'][0]['name'])
            return libs
        except:
            stdinit.std_signal_gobal.stdprintln()


    def lib_search(self,p):#page
        try:

            cmd = self.pio_env+" lib search --page " + str(p) + " --json-output"
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            try:
                data = str(rest.stdout.read(), encoding='gbk')
            except:
                data = str(rest.stdout.read())

            rest.stdout.close()
            libs = json.loads(data)  # 字符串转json
            # print(libs['items'][0]['name'])
            return libs
        except:
            stdinit.std_signal_gobal.stdprintln()

        # print("data2['name']: ", libs[0]['ownername'])
    def is_installed_lib(self):

        pass



    def lib_list(self):  # installed
        try:

            cmd = self.pio_env+" lib list --json-output"
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            try:
                data = str(rest.stdout.read(), encoding='gbk')
            except:
                data= str(rest.stdout.read())
            rest.stdout.close()
            pioplatforms = json.loads(data)  # 字符串转json
            return pioplatforms
            # print("data2['name']: ", self.pioplatforms[0]['ownername'])

            pass
        except:
            stdinit.std_signal_gobal.stdprintln()


    def lib_show(self, name):  # installed
        try:

            cmd = self.pio_env+" lib show " + name
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            data = str(rest.stdout.read(), encoding='gbk')
            rest.stdout.close()
            pioplatform = json.loads(data)  # 字符串转json
            return pioplatform
            # print("data2['name']: ", self.pioplatforms[0]['ownername'])
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()


    def lib_install(self, id):#pro_path
        try:

            cmd = self.pio_env+" lib -d " + '"' +stdinit.projects_dir + stdinit.project_name+ '"'  + " install " + id
            # res = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)
                stdinit.std_signal_gobal.std_echo_msg(1, s1)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
        except:
            stdinit.std_signal_gobal.stdprintln()



    def lib_uninstall(self, id):
        try:
            cmd = self.pio_env+" lib -d " + '"' +stdinit.projects_dir + stdinit.project_name+ '"'  + " uninstall " + id
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)
                stdinit.std_signal_gobal.std_echo_msg(1, s1)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
        except:
            stdinit.std_signal_gobal.stdprintln()


    def lib_update(self,pro_path):  # 待完善
        try:
            if self.check_net()==False:
                return False
            cmd = self.pio_env+" lib --storage-dir " + '"' +pro_path + '"' + " update"
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)

                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
        except:
            stdinit.std_signal_gobal.stdprintln()

    def lib_all_update(self):  # 待完善Update all installed libraries in global storage
        try:
            if self.check_net()==False:
                return False
            cmd = self.pio_env+" lib -g update"
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
        except:
            stdinit.std_signal_gobal.stdprintln()


    # pio platform install [OPTIONS] [PLATFORM...]
    #
    # # [PLATFORM...] forms
    # pio platform install <name>
    # pio platform install <name>@<version>
    # pio platform install <name>@<version range>
    # pio platform install <ownername>@<version>
    # pio platform install <ownername>@<version range>
    # pio platform install <zip or tarball url>
    # pio platform install file://<zip or tarball file>
    # pio platform install file://<folder>
    # pio platform install <repository>
    # pio platform install <name=repository> (name it should have locally)
    # pio platform install <repository#tag> ("tag" can be commit, branch or tag)



# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
import subprocess
import os,stat
from ..cores.stdedit import stdinit
#from function.cores.stdedit import stdinit
from ..conf import res, setup
#from function.conf import res, setup
from shutil import copytree,rmtree,copy2
import threading
import requests
import json
import zipfile
import time
#调用前先判断是否已经正常安装pio
class PioInstall():
    def __init__(self):
        stdinit.std_signal_gobal.stdenvir_init.connect(self.all_init)
        if stdinit.platform_is == "Win":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio" + '"'

        elif stdinit.platform_is == "Linux":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'

        elif stdinit.platform_is == "Darwin":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'


        #print(self.abs_path)
        #self.pip_fast()

    def pip_fast(self):#all the time
        try:
            if res.msg == "1":  # 中
                target = stdinit.stdenv + "/pip"
                # target="C:/stwork/stdemo2019827/tool/packages/pip2/pip.ini"
                if os.path.exists(target):
                    pass
                else:
                    try:
                        source = stdinit.abs_path + "/tool/packages/pip"
                        # print(source)
                        # target=os.environ["USERPROFILE"] + "/pip/pip.ini"
                        copytree(source, target)
                    except:
                        stdinit.std_signal_gobal.stdprintln()
                    pass
                pass
        except:
            stdinit.std_signal_gobal.stdprintln()

    def is_installed_pio(self):#mac 和linux 好像不同
        try:



            target = stdinit.stdenv + "/.stduino/packages/pioenv"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
            #print(target)
            if os.path.exists(target):
                return True
            else:
                return False
        except:

            stdinit.std_signal_gobal.stdprintln()


    def is_init_pioenv(self):
        try:
            target = stdinit.stdenv + "/.stduino/packages/pioenv"
            if os.path.exists(target):
                return True
            else:
                return False
        except:
            stdinit.std_signal_gobal.stdprintln()


    def pioenv_init(self):

        try:
            if stdinit.platform_is == "Win":
                cmd = stdinit.stdenv + "/.stduino/packages/python3/python -m venv " + '"' +stdinit.stdenv + "/.stduino/packages/pioenv"+ '"'

            elif stdinit.platform_is == "Linux":
                cmd = stdinit.stdenv + "/.stduino/packages/python3/bin/virtualenv " + '"' +stdinit.stdenv + "/.stduino/packages/pioenv"+ '"'

            elif stdinit.platform_is == "Darwin":
                cmd = stdinit.stdenv + "/.stduino/packages/python3/bin/python3 -m venv " + '"' + stdinit.stdenv + "/.stduino/packages/pioenv" + '"'

            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)

                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
        except:
            stdinit.std_signal_gobal.stdprintln()
    def check_net(self):
        try:
            return stdinit.std_signal_gobal.is_connected()

        except:
            return False


            # if stdinit.platform_is == "Win":
            #     request.urlopen(url="https://www.baidu.com", timeout=3.0)
            #
            #
            # elif stdinit.platform_is == "Linux":
            #     pass
            #
            # elif stdinit.platform_is == "Darwin":
            #     pass

            # print(ret)


    def install_pio(self):
        try:
            if stdinit.platform_is == "Win":
                env_path = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pip" + '"'# install -i https://mirrors.bfsu.edu.cn/pypi/web/simple -U platformio
                cmd = env_path + " install -i https://mirrors.bfsu.edu.cn/pypi/web/simple platformio==6.1.4"#-U 已安装就升级到最新版
                proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
                for line in iter(proc.stdout.readline, b''):
                    try:
                        s1 = str(line, encoding='gbk')
                    except:
                        s1 =  str(line)
                    # print(s1)
                    if not subprocess.Popen.poll(proc) is None:
                        if line == "":
                            break
                proc.stdout.close()  # pioenv\Lib\site-packages\platformio\package\manager
                regi = stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/platformio/package/manager/_registry.py"
                self.delete_file(regi)
                copy2(stdinit.abs_path + "/tool/packages/python3/Scripts/stdload.py", regi)

                copy2(stdinit.abs_path + "/tool/packages/python3/Scripts/stdini.py",
                      stdinit.stdenv + "/.stduino/packages/pioenv/Lib/site-packages/platformio/ideini.pp")

            elif stdinit.platform_is == "Linux":
                env_path= '"' +stdinit.stdenv + "/.stduino/packages/pioenv/bin/pip"+ '"'
                cmd = env_path + " install -i https://mirrors.bfsu.edu.cn/pypi/web/simple platformio==6.1.4"
                proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
                for line in iter(proc.stdout.readline, b''):
                    try:
                        s1 = str(line, encoding='gbk')
                    except:
                        s1 =  str(line)
                    # print(s1)
                    if not subprocess.Popen.poll(proc) is None:
                        if line == "":
                            break
                proc.stdout.close()  # pioenv\Lib\site-packages\platformio\package\manager
                regi = stdinit.stdenv + "/.stduino/packages/pioenv/lib/python3.6/site-packages/platformio/package/manager/_registry.py"
                self.delete_file(regi)
                copy2(stdinit.abs_path + "/tool/packages/python3/bin/stdload.py", regi)

                copy2(stdinit.abs_path + "/tool/packages/python3/bin/stdini.py",
                      stdinit.stdenv + "/.stduino/packages/pioenv/lib/python3.6/site-packages/platformio/ideini.pp")

            elif stdinit.platform_is == "Darwin":
                env_path= '"' +stdinit.stdenv + "/.stduino/packages/pioenv/bin/pip"+ '"'
                cmd = env_path + " install -i https://mirrors.bfsu.edu.cn/pypi/web/simple platformio==6.1.4"
                proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
                for line in iter(proc.stdout.readline, b''):
                    try:
                        s1 = str(line, encoding='gbk')
                    except:
                        s1 = str(line)
                    # print(s1)
                    if not subprocess.Popen.poll(proc) is None:
                        if line == "":
                            break
                proc.stdout.close()  # pioenv\Lib\site-packages\platformio\package\manager
                regi = stdinit.stdenv + "/.stduino/packages/pioenv/lib/python3.9/site-packages/platformio/package/manager/_registry.py"

                self.delete_file(regi)

                copy2(stdinit.abs_path + "/tool/packages/python3/bin/stdload.py", regi)

                copy2(stdinit.abs_path + "/tool/packages/python3/bin/stdini.py",
                      stdinit.stdenv + "/.stduino/packages/pioenv/lib/python3.9/site-packages/platformio/ideini.pp")


            # if res.msg == "1":  # 中
            #     cmd = stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pip install -i https://pypi.tuna.tsinghua.edu.cn/simple -U platformio"
            # else:
            #     cmd = stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pip install -U platformio"

            copy2(stdinit.abs_path + "/tool/packages/pioboards.json",
                  stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json")
            copy2(stdinit.abs_path + "/tool/packages/stdpio.ini",
                  stdinit.stdenv + "/.stduino/packages/pioenv/stdpio.ini")
            return True
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False
    def get_board_v(self):
        try:
            if ((int(time.time()) - int(res.updatetime)) > 586400):
                setup.updatetime(str(int(time.time())))
                r = requests.get("http://api.stduino.com/getvboards")
                if (r.status_code == 200):
                    version_data = json.loads(r.text)
                    if (int(version_data['version_id']) != int(res.version_id)):
                        url = 'https://stduino-generic.pkg.coding.net/stduino_packages/boards/boards.zip?version=latest'
                        r = requests.get(url)
                        bzip_path=stdinit.stdenv + "/.stduino/packages/boards.zip"
                        with open(bzip_path, "wb") as code:
                            code.write(r.content)
                        url = 'https://stduino-generic.pkg.coding.net/stduino_packages/boards/Custom.zip?version=latest'
                        r = requests.get(url)
                        czip_path = stdinit.stdenv + "/.stduino/packages/Custom.zip"
                        with open(czip_path, "wb") as code:
                            code.write(r.content)
                        try:
                            board_p = stdinit.stdenv + "/.platformio/boards"
                            core_p = stdinit.stdenv + "/.platformio/"
                            board_pin_p = stdinit.stdenv + "/.platformio/packages/framework-arduinoststm32/variants/Custom"
                            core_pin_p = stdinit.stdenv + "/.platformio/packages/framework-arduinoststm32/variants/"
                            if os.path.exists(board_p):
                                self.delete_file(board_p)

                            with zipfile.ZipFile(bzip_path, mode="r") as f:
                                f.extractall(core_p)  # .platformio ##将文件解压到指定目录，解压密码为root
                            if os.path.exists(board_pin_p):
                                self.delete_file(board_pin_p)

                            with zipfile.ZipFile(czip_path, mode="r") as f:
                                f.extractall(core_pin_p)  # .platformio ##将文件解压到指定目录，解压密码为root
                        except:
                            f.close()
                            stdinit.std_signal_gobal.stdprintln()

                        if os.path.exists(bzip_path):
                            self.delete_file(bzip_path)
                        if os.path.exists(czip_path):
                            self.delete_file(czip_path)
                        setup.version_id(str(version_data['version_id']))
                pass

        except:
            stdinit.std_signal_gobal.stdprintln()


    def generate_project_main(self):
        try:
            main_content = "\n".join(
                [
                    "#include <Arduino.h>",
                    "",
                    "void setup() {",
                    "  // put your setup code here, to run once:",
                    "}",
                    "",
                    "void loop() {",
                    "  // put your main code here, to run repeatedly:",
                    "}",
                    "",
                ]
            )

            src_dir = stdinit.projects_dir +  "stdtest" + "/src"
            main_path = stdinit.projects_dir + "stdtest"+ "/src/main.cpp"
            if os.path.isfile(main_path):
                return True
            if os.path.exists(src_dir):
                pass
            else:
                os.makedirs(src_dir)
            with open(main_path, "w") as fp:
                fp.write(main_content.strip())
                fp.close()
            return True
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False
    def std_board_init(self):
        try:
            url = 'https://stduino-generic.pkg.coding.net/stduino_packages/boards/boards.zip?version=latest'
            r = requests.get(url)
            bzip_path = stdinit.stdenv + "/.stduino/packages/boards.zip"
            with open(bzip_path, "wb") as code:
                code.write(r.content)
            url = 'https://stduino-generic.pkg.coding.net/stduino_packages/boards/Custom.zip?version=latest'
            r = requests.get(url)
            czip_path = stdinit.stdenv + "/.stduino/packages/Custom.zip"
            with open(czip_path, "wb") as code:
                code.write(r.content)
            try:
                board_p = stdinit.stdenv + "/.platformio/boards"
                core_p = stdinit.stdenv + "/.platformio/"
                board_pin_p = stdinit.stdenv + "/.platformio/packages/framework-arduinoststm32/variants/Custom"
                core_pin_p = stdinit.stdenv + "/.platformio/packages/framework-arduinoststm32/variants/"
                if os.path.exists(board_p):
                    self.delete_file(board_p)

                with zipfile.ZipFile(bzip_path, mode="r") as f:
                    f.extractall(core_p)  # .platformio ##将文件解压到指定目录，解压密码为root
                if os.path.exists(board_pin_p):
                    self.delete_file(board_pin_p)

                with zipfile.ZipFile(czip_path, mode="r") as f:
                    f.extractall(core_pin_p)  # .platformio ##将文件解压到指定目录，解压密码为root
            except:
                stdinit.std_signal_gobal.stdprintln()
            f.close()
            if os.path.exists(bzip_path):
                self.delete_file(bzip_path)
            if os.path.exists(czip_path):
                self.delete_file(czip_path)
            r = requests.get("http://api.stduino.com/getvboards")
            if (r.status_code == 200):
                version_data = json.loads(r.text)
                setup.version_id(str(version_data['version_id']))
                setup.updatetime(str(int(time.time())))

        except: 
            stdinit.std_signal_gobal.stdprintln()


    def std_init(self):
        try:
            init_ok = 0
            target_path = stdinit.projects_dir + "stdtest"
            if os.path.exists(target_path):
                self.delete_file(target_path)
                os.makedirs(target_path)
            else:
                os.makedirs(target_path)
            # cmd = self.pio_env + " platform install ststm32"
            # proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            # for line in iter(proc.stdout.readline, b''):
            #     try:
            #         s1 = str(line, encoding='gbk')
            #     except:
            #         s1 = str(line)
            #     # print(s1)
            #     if not subprocess.Popen.poll(proc) is None:
            #         if line == "":
            #             break
            # proc.stdout.close()  # pioenv\Lib\site-packages\platformio\package\manager
            cmd = self.pio_env + " project init -d " + target_path + " --board bluepill_f103c8 -O framework=arduino -O debug_tool=blackmagic"
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)
                #print(s1)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
            proc.stdout.close()  # pioenv\Lib\site-packages\platformio\package\manager
            self.generate_project_main()
            # ststm32 project init
            # cmd="C:/Users/debug/.stduino/packages/pioenv/Scripts/pio" project init -d "C:/Users/debug/Documents/Stduino/Projects/stdtest" --board bluepill_f103c8 -O framework=arduino -O debug_tool=blackmagic
            pass
            cmd = self.pio_env + " run -d " + target_path
            stdinit.std_signal_gobal.std_process(1, "环境初始化")  # complexing
            stdinit.std_signal_gobal.std_echo_msg(0, "第一次启动，软件正在初始化,请稍等片刻~")
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='gbk')
                    #print(s1)
                    # if "SUCCESS" in s1:
                    #     init_ok=1
                except:
                    s1 = str(line)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
            proc.stdout.close()  # pioenv\Lib\site-packages\platformio\package\manager
            if init_ok == 1:
                pass
            self.std_board_init()
            stdinit.std_signal_gobal.std_process(0, "初始化完成")  # complexing
            stdinit.std_signal_gobal.std_echo_msg(0, "初始化完成，若在使用过程中遇到问题，欢迎随时至www.stduino.com进行发帖交流！")

            # copy2(stdinit.abs_path + "/tool/packages/stdinit/pioboards.json",
            #       stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json")
            # copy2(stdinit.abs_path + "/tool/packages/stdpio.ini",
            #       stdinit.stdenv + "/.stduino/packages/pioenv/stdpio.ini")

            # "C:/Users/debug/.stduino/packages/pioenv/Scripts/pio" run -d "C:/Users/debug/Documents/Stduino/Projects/stdtest"
            # project run
            # Building.pio\build\bluepill_f103c8\firmware.bin
            # SUCCESS
            pass
            # copy stdfile——》plaforms&packages
            pass
        except:
            stdinit.std_signal_gobal.std_process(0, "初始化失败")  # complexing
            stdinit.std_signal_gobal.std_echo_msg(0, "初始化失败，请重启后再次进行初始化，欢迎随时至www.stduino.com进行发帖交流！")


    def all_inits(self):
        try:
            if self.check_net() == False:
                return False
            stm32_platpath = stdinit.stdenv + "/.platformio/platforms/ststm32"
            stm32_frampath = stdinit.stdenv + "/.platformio/packages/framework-arduinoststm32"
            if self.pio_install():
                if os.path.exists(stm32_platpath):
                    if os.path.exists(stm32_frampath):
                        board_p = stdinit.stdenv + "/.platformio/boards"
                        if os.path.exists(board_p):
                            self.get_board_v()
                        else:
                            self.std_board_init()
                    else:
                        self.std_init()
                    pass
                else:
                    self.std_init()
            self.get_board_v()
        except:
            pass

    def all_init(self):
        t1 = threading.Thread(target=self.all_inits, name='all_inits')
        t1.setDaemon(True)
        t1.start()  # File_tree_init



    #     if os.path.exists(stdenv):
    #         pass
    #     else:
    #         os.makedirs(stdenv)
    def pio_install(self):
        try:

            if self.is_installed_pio() == False:

                if self.check_net()==False:
                    return False

                if self.is_init_pioenv() == False:

                    self.pioenv_init()
                    self.install_pio()
                    return True
                else:
                    self.install_pio()
                    return True

            return True
        except:
            stdinit.std_signal_gobal.stdprintln()
            return False

        #self.pip_fast()
        # if os.environ["USERPROFILE"]+"/pip":

        #     pass

    def remove_readonly(self,func, path, _):
        try:
            # "Clear the readonly bit and reattempt the removal"
            os.chmod(path, stat.S_IWRITE)
            func(path)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def delete_file(self,path):
        try:
            if os.path.isdir(path):
                try:
                    rmtree(path, onerror=self.remove_readonly)
                except:
                    stdinit.std_signal_gobal.stdprintln()
                    # self.listwidget.append("Delete error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")
                    # self.add_lib_si(python3, 0, "Unexpected error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder")
                    # QMessageBox.warning(self, "Delete error",
                    #                     "Unexpected error:" + "\n 请手动至该文件夹处删除\nPlease manually delete to this folder",
                    #                     QMessageBox.Yes)
                    pass
            else:
                try:
                    os.remove(path)
                except:
                    try:
                        # shutil.rmtree(path, onerror=self.remove_readonly)
                        os.chmod(path, stat.S_IWRITE)
                        os.remove(path)
                    except:
                        stdinit.std_signal_gobal.stdprintln()
                pass
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()

    def pio_uninstall(self):
        try:
            if self.is_init_pioenv() == False:
                pass
            else:
                try:
                    self.delete_file(stdinit.stdenv + "/.stduino/packages/pioenv")
                    return True
                except:
                    stdinit.std_signal_gobal.stdprintln()
                    return False

        except:
            stdinit.std_signal_gobal.stdprintln()

    def pio_test(self):
        try:
            print(stdinit.board)
            stdinit.board = "ddds"
        except:
            stdinit.std_signal_gobal.stdprintln()


if __name__ == '__main__':  # main函数
    pass
    #PioInstall() langeage 工作目录需在test里进行测试
    # get_board_v()





